/*
 * Copyright (C) 2018-2019 QuasarApp.
 * Distributed under the lgplv3 software license, see the accompanying
 * Everyone is permitted to copy and distribute verbatim copies
 * of this license document, but changing it is not allowed.
 */

#include "modules.h"
#include <configparser.h>

Modules::Modules()
{

}

QSet<QString> Modules::qtLibs() {
    TestUtils utils;
#ifdef Q_OS_WIN
    auto Tree = utils.createTree(
    {
                    "./" + DISTRO_DIR + "/Qt5Core.dll",
                    "./" + DISTRO_DIR + "/Qt5Gui.dll",
                    "./" + DISTRO_DIR + "/Qt5Network.dll",
                    "./" + DISTRO_DIR + "/Qt5Qml.dll",
                    "./" + DISTRO_DIR + "/Qt5Quick.dll",
                    "./" + DISTRO_DIR + "/Qt5Svg.dll",
                    "./" + DISTRO_DIR + "/Qt5VirtualKeyboard.dll",
                    "./" + DISTRO_DIR + "/Qt5WebSockets.dll",
                    "./" + DISTRO_DIR + "/Qt5Widgets.dll",
                    "./" + DISTRO_DIR + "/QtWidgetsProject.exe",
                    "./" + DISTRO_DIR + "/libgcc_s_seh-1.dll",
                    "./" + DISTRO_DIR + "/libstdc++-6.dll",
                    "./" + DISTRO_DIR + "/libwinpthread-1.dll",
                    "./" + DISTRO_DIR + "/plugins/iconengines/qsvgicon.dll",
                    "./" + DISTRO_DIR + "/plugins/imageformats/qgif.dll",
                    "./" + DISTRO_DIR + "/plugins/imageformats/qicns.dll",
                    "./" + DISTRO_DIR + "/plugins/imageformats/qico.dll",
                    "./" + DISTRO_DIR + "/plugins/imageformats/qjpeg.dll",
                    "./" + DISTRO_DIR + "/plugins/imageformats/qsvg.dll",
                    "./" + DISTRO_DIR + "/plugins/imageformats/qtga.dll",
                    "./" + DISTRO_DIR + "/plugins/imageformats/qtiff.dll",
                    "./" + DISTRO_DIR + "/plugins/imageformats/qwbmp.dll",
                    "./" + DISTRO_DIR + "/plugins/imageformats/qwebp.dll",
                    "./" + DISTRO_DIR + "/plugins/platforminputcontexts/qtvirtualkeyboardplugin.dll",
                    "./" + DISTRO_DIR + "/plugins/platforms/qminimal.dll",
                    "./" + DISTRO_DIR + "/plugins/platforms/qoffscreen.dll",
                    "./" + DISTRO_DIR + "/plugins/platforms/qwebgl.dll",
                    "./" + DISTRO_DIR + "/plugins/platforms/qwindows.dll",
                    "./" + DISTRO_DIR + "/plugins/styles/qwindowsvistastyle.dll",
                    "./" + DISTRO_DIR + "/plugins/virtualkeyboard/qtvirtualkeyboard_hangul.dll",
                    "./" + DISTRO_DIR + "/plugins/virtualkeyboard/qtvirtualkeyboard_openwnn.dll",
                    "./" + DISTRO_DIR + "/plugins/virtualkeyboard/qtvirtualkeyboard_pinyin.dll",
                    "./" + DISTRO_DIR + "/plugins/virtualkeyboard/qtvirtualkeyboard_tcime.dll",
                    "./" + DISTRO_DIR + "/plugins/virtualkeyboard/qtvirtualkeyboard_thai.dll",
                    "./" + DISTRO_DIR + "/qt.conf",
                    "./" + DISTRO_DIR + "/translations/qtbase_ar.qm",
                    "./" + DISTRO_DIR + "/translations/qtbase_bg.qm",
                    "./" + DISTRO_DIR + "/translations/qtbase_ca.qm",
                    "./" + DISTRO_DIR + "/translations/qtbase_cs.qm",
                    "./" + DISTRO_DIR + "/translations/qtbase_da.qm",
                    "./" + DISTRO_DIR + "/translations/qtbase_de.qm",
                    "./" + DISTRO_DIR + "/translations/qtbase_en.qm",
                    "./" + DISTRO_DIR + "/translations/qtbase_es.qm",
                    "./" + DISTRO_DIR + "/translations/qtbase_fi.qm",
                    "./" + DISTRO_DIR + "/translations/qtbase_fr.qm",
                    "./" + DISTRO_DIR + "/translations/qtbase_gd.qm",
                    "./" + DISTRO_DIR + "/translations/qtbase_he.qm",
                    "./" + DISTRO_DIR + "/translations/qtbase_hu.qm",
                    "./" + DISTRO_DIR + "/translations/qtbase_it.qm",
                    "./" + DISTRO_DIR + "/translations/qtbase_ja.qm",
                    "./" + DISTRO_DIR + "/translations/qtbase_ko.qm",
                    "./" + DISTRO_DIR + "/translations/qtbase_lv.qm",
                    "./" + DISTRO_DIR + "/translations/qtbase_pl.qm",
                    "./" + DISTRO_DIR + "/translations/qtbase_ru.qm",
                    "./" + DISTRO_DIR + "/translations/qtbase_sk.qm",
                    "./" + DISTRO_DIR + "/translations/qtbase_uk.qm",
                    "./" + DISTRO_DIR + "/translations/qtbase_zh_TW.qm",
                    "./" + DISTRO_DIR + "/translations/qtdeclarative_bg.qm",
                    "./" + DISTRO_DIR + "/translations/qtdeclarative_da.qm",
                    "./" + DISTRO_DIR + "/translations/qtdeclarative_de.qm",
                    "./" + DISTRO_DIR + "/translations/qtdeclarative_en.qm",
                    "./" + DISTRO_DIR + "/translations/qtdeclarative_es.qm",
                    "./" + DISTRO_DIR + "/translations/qtdeclarative_fi.qm",
                    "./" + DISTRO_DIR + "/translations/qtdeclarative_fr.qm",
                    "./" + DISTRO_DIR + "/translations/qtdeclarative_hu.qm",
                    "./" + DISTRO_DIR + "/translations/qtdeclarative_ja.qm",
                    "./" + DISTRO_DIR + "/translations/qtdeclarative_ko.qm",
                    "./" + DISTRO_DIR + "/translations/qtdeclarative_lv.qm",
                    "./" + DISTRO_DIR + "/translations/qtdeclarative_pl.qm",
                    "./" + DISTRO_DIR + "/translations/qtdeclarative_ru.qm",
                    "./" + DISTRO_DIR + "/translations/qtdeclarative_sk.qm",
                    "./" + DISTRO_DIR + "/translations/qtdeclarative_uk.qm"
                });

#else
    auto Tree = utils.createTree(
    {
                    "./" + DISTRO_DIR + "/bin/qt.conf",
                    "./" + DISTRO_DIR + "/QtWidgetsProject.sh",
                    "./" + DISTRO_DIR + "/bin/QtWidgetsProject",
                    "./" + DISTRO_DIR + "/lib/libQt5Core.so",
                    "./" + DISTRO_DIR + "/lib/libQt5DBus.so",
                    "./" + DISTRO_DIR + "/lib/libQt5EglFSDeviceIntegration.so",
                    "./" + DISTRO_DIR + "/lib/libQt5Gui.so",
                    "./" + DISTRO_DIR + "/lib/libQt5Network.so",
                    "./" + DISTRO_DIR + "/lib/libQt5Qml.so",
                    "./" + DISTRO_DIR + "/lib/libQt5Quick.so",
                    "./" + DISTRO_DIR + "/lib/libQt5Svg.so",
                    "./" + DISTRO_DIR + "/lib/libQt5VirtualKeyboard.so",
                    "./" + DISTRO_DIR + "/lib/libQt5WaylandClient.so",
                    "./" + DISTRO_DIR + "/lib/libQt5WaylandCompositor.so",
                    "./" + DISTRO_DIR + "/lib/libQt5WebSockets.so",
                    "./" + DISTRO_DIR + "/lib/libQt5Widgets.so",
                    "./" + DISTRO_DIR + "/lib/libQt5XcbQpa.so",
                    "./" + DISTRO_DIR + "/lib/libicudata.so",
                    "./" + DISTRO_DIR + "/lib/libicui18n.so",
                    "./" + DISTRO_DIR + "/lib/libicuuc.so",
                    "./" + DISTRO_DIR + "/plugins/iconengines/libqsvgicon.so",
                    "./" + DISTRO_DIR + "/plugins/imageformats/libqgif.so",
                    "./" + DISTRO_DIR + "/plugins/imageformats/libqicns.so",
                    "./" + DISTRO_DIR + "/plugins/imageformats/libqico.so",
                    "./" + DISTRO_DIR + "/plugins/imageformats/libqjpeg.so",
                    "./" + DISTRO_DIR + "/plugins/imageformats/libqsvg.so",
                    "./" + DISTRO_DIR + "/plugins/imageformats/libqtga.so",
                    "./" + DISTRO_DIR + "/plugins/imageformats/libqtiff.so",
                    "./" + DISTRO_DIR + "/plugins/imageformats/libqwbmp.so",
                    "./" + DISTRO_DIR + "/plugins/imageformats/libqwebp.so",
                    "./" + DISTRO_DIR + "/plugins/platforminputcontexts/libcomposeplatforminputcontextplugin.so",
                    "./" + DISTRO_DIR + "/plugins/platforminputcontexts/libibusplatforminputcontextplugin.so",
                    "./" + DISTRO_DIR + "/plugins/platforminputcontexts/libqtvirtualkeyboardplugin.so",
                    "./" + DISTRO_DIR + "/plugins/platforms/libqeglfs.so",
                    "./" + DISTRO_DIR + "/plugins/platforms/libqlinuxfb.so",
                    "./" + DISTRO_DIR + "/plugins/platforms/libqminimal.so",
                    "./" + DISTRO_DIR + "/plugins/platforms/libqminimalegl.so",
                    "./" + DISTRO_DIR + "/plugins/platforms/libqoffscreen.so",
                    "./" + DISTRO_DIR + "/plugins/platforms/libqvnc.so",
                    "./" + DISTRO_DIR + "/plugins/platforms/libqwayland-egl.so",
                    "./" + DISTRO_DIR + "/plugins/platforms/libqwayland-generic.so",
                    "./" + DISTRO_DIR + "/plugins/platforms/libqwayland-xcomposite-egl.so",
                    "./" + DISTRO_DIR + "/plugins/platforms/libqwayland-xcomposite-glx.so",
                    "./" + DISTRO_DIR + "/plugins/platforms/libqwebgl.so",
                    "./" + DISTRO_DIR + "/plugins/platforms/libqxcb.so",
                    "./" + DISTRO_DIR + "/plugins/virtualkeyboard/libqtvirtualkeyboard_hangul.so",
                    "./" + DISTRO_DIR + "/plugins/virtualkeyboard/libqtvirtualkeyboard_openwnn.so",
                    "./" + DISTRO_DIR + "/plugins/virtualkeyboard/libqtvirtualkeyboard_pinyin.so",
                    "./" + DISTRO_DIR + "/plugins/virtualkeyboard/libqtvirtualkeyboard_tcime.so",
                    "./" + DISTRO_DIR + "/plugins/virtualkeyboard/libqtvirtualkeyboard_thai.so",
                    "./" + DISTRO_DIR + "/plugins/wayland-decoration-client/libbradient.so",
                    "./" + DISTRO_DIR + "/plugins/wayland-graphics-integration-client/libdmabuf-server.so",
                    "./" + DISTRO_DIR + "/plugins/wayland-graphics-integration-client/libdrm-egl-server.so",
                    "./" + DISTRO_DIR + "/plugins/wayland-graphics-integration-client/libqt-plugin-wayland-egl.so",
                    "./" + DISTRO_DIR + "/plugins/wayland-graphics-integration-client/libshm-emulation-server.so",
                    "./" + DISTRO_DIR + "/plugins/wayland-graphics-integration-client/libxcomposite-egl.so",
                    "./" + DISTRO_DIR + "/plugins/wayland-graphics-integration-client/libxcomposite-glx.so",
                    "./" + DISTRO_DIR + "/plugins/wayland-graphics-integration-server/libdmabuf-server.so",
                    "./" + DISTRO_DIR + "/plugins/wayland-graphics-integration-server/libdrm-egl-server.so",
                    "./" + DISTRO_DIR + "/plugins/wayland-graphics-integration-server/liblinux-dmabuf-unstable-v1.so",
                    "./" + DISTRO_DIR + "/plugins/wayland-graphics-integration-server/libqt-plugin-wayland-egl.so",
                    "./" + DISTRO_DIR + "/plugins/wayland-graphics-integration-server/libshm-emulation-server.so",
                    "./" + DISTRO_DIR + "/plugins/wayland-graphics-integration-server/libwayland-eglstream-controller.so",
                    "./" + DISTRO_DIR + "/plugins/wayland-graphics-integration-server/libxcomposite-egl.so",
                    "./" + DISTRO_DIR + "/plugins/wayland-graphics-integration-server/libxcomposite-glx.so",
                    "./" + DISTRO_DIR + "/plugins/wayland-shell-integration/libfullscreen-shell-v1.so",
                    "./" + DISTRO_DIR + "/plugins/wayland-shell-integration/libivi-shell.so",
                    "./" + DISTRO_DIR + "/plugins/wayland-shell-integration/libwl-shell.so",
                    "./" + DISTRO_DIR + "/plugins/wayland-shell-integration/libxdg-shell-v5.so",
                    "./" + DISTRO_DIR + "/plugins/wayland-shell-integration/libxdg-shell-v6.so",
                    "./" + DISTRO_DIR + "/plugins/wayland-shell-integration/libxdg-shell.so",
                    "./" + DISTRO_DIR + "/plugins/xcbglintegrations/libqxcb-egl-integration.so",
                    "./" + DISTRO_DIR + "/plugins/xcbglintegrations/libqxcb-glx-integration.so",
                    "./" + DISTRO_DIR + "/translations/qtbase_ar.qm",
                    "./" + DISTRO_DIR + "/translations/qtbase_bg.qm",
                    "./" + DISTRO_DIR + "/translations/qtbase_ca.qm",
                    "./" + DISTRO_DIR + "/translations/qtbase_cs.qm",
                    "./" + DISTRO_DIR + "/translations/qtbase_da.qm",
                    "./" + DISTRO_DIR + "/translations/qtbase_de.qm",
                    "./" + DISTRO_DIR + "/translations/qtbase_en.qm",
                    "./" + DISTRO_DIR + "/translations/qtbase_es.qm",
                    "./" + DISTRO_DIR + "/translations/qtbase_fi.qm",
                    "./" + DISTRO_DIR + "/translations/qtbase_fr.qm",
                    "./" + DISTRO_DIR + "/translations/qtbase_gd.qm",
                    "./" + DISTRO_DIR + "/translations/qtbase_he.qm",
                    "./" + DISTRO_DIR + "/translations/qtbase_hu.qm",
                    "./" + DISTRO_DIR + "/translations/qtbase_it.qm",
                    "./" + DISTRO_DIR + "/translations/qtbase_ja.qm",
                    "./" + DISTRO_DIR + "/translations/qtbase_ko.qm",
                    "./" + DISTRO_DIR + "/translations/qtbase_lv.qm",
                    "./" + DISTRO_DIR + "/translations/qtbase_pl.qm",
                    "./" + DISTRO_DIR + "/translations/qtbase_ru.qm",
                    "./" + DISTRO_DIR + "/translations/qtbase_sk.qm",
                    "./" + DISTRO_DIR + "/translations/qtbase_uk.qm",
                    "./" + DISTRO_DIR + "/translations/qtbase_zh_TW.qm",
                    "./" + DISTRO_DIR + "/translations/qtdeclarative_bg.qm",
                    "./" + DISTRO_DIR + "/translations/qtdeclarative_da.qm",
                    "./" + DISTRO_DIR + "/translations/qtdeclarative_de.qm",
                    "./" + DISTRO_DIR + "/translations/qtdeclarative_en.qm",
                    "./" + DISTRO_DIR + "/translations/qtdeclarative_es.qm",
                    "./" + DISTRO_DIR + "/translations/qtdeclarative_fi.qm",
                    "./" + DISTRO_DIR + "/translations/qtdeclarative_fr.qm",
                    "./" + DISTRO_DIR + "/translations/qtdeclarative_hu.qm",
                    "./" + DISTRO_DIR + "/translations/qtdeclarative_ja.qm",
                    "./" + DISTRO_DIR + "/translations/qtdeclarative_ko.qm",
                    "./" + DISTRO_DIR + "/translations/qtdeclarative_lv.qm",
                    "./" + DISTRO_DIR + "/translations/qtdeclarative_pl.qm",
                    "./" + DISTRO_DIR + "/translations/qtdeclarative_ru.qm",
                    "./" + DISTRO_DIR + "/translations/qtdeclarative_sk.qm",
                    "./" + DISTRO_DIR + "/translations/qtdeclarative_uk.qm"
                }
                );
#endif
    return Tree;
}

QSet<QString> Modules::qmlLibs() {
    TestUtils utils;

#ifdef Q_OS_WIN
    auto tree = utils.createTree(
    {
                    "./" + DISTRO_DIR + "/Qt5Core.dll",
                    "./" + DISTRO_DIR + "/Qt5Gui.dll",
                    "./" + DISTRO_DIR + "/Qt5Network.dll",
                    "./" + DISTRO_DIR + "/Qt5Qml.dll",
                    "./" + DISTRO_DIR + "/Qt5Quick.dll",
                    "./" + DISTRO_DIR + "/Qt5QuickControls2.dll",
                    "./" + DISTRO_DIR + "/Qt5QuickTemplates2.dll",
                    "./" + DISTRO_DIR + "/Qt5Svg.dll",
                    "./" + DISTRO_DIR + "/Qt5VirtualKeyboard.dll",
                    "./" + DISTRO_DIR + "/Qt5WebSockets.dll",
                    "./" + DISTRO_DIR + "/Qt5Widgets.dll",
                    "./" + DISTRO_DIR + "/TestQMLWidgets.exe",
                    "./" + DISTRO_DIR + "/libgcc_s_seh-1.dll",
                    "./" + DISTRO_DIR + "/libstdc++-6.dll",
                    "./" + DISTRO_DIR + "/libwinpthread-1.dll",
                    "./" + DISTRO_DIR + "/plugins/bearer/qgenericbearer.dll",
                    "./" + DISTRO_DIR + "/plugins/iconengines/qsvgicon.dll",
                    "./" + DISTRO_DIR + "/plugins/imageformats/qgif.dll",
                    "./" + DISTRO_DIR + "/plugins/imageformats/qicns.dll",
                    "./" + DISTRO_DIR + "/plugins/imageformats/qico.dll",
                    "./" + DISTRO_DIR + "/plugins/imageformats/qjpeg.dll",
                    "./" + DISTRO_DIR + "/plugins/imageformats/qsvg.dll",
                    "./" + DISTRO_DIR + "/plugins/imageformats/qtga.dll",
                    "./" + DISTRO_DIR + "/plugins/imageformats/qtiff.dll",
                    "./" + DISTRO_DIR + "/plugins/imageformats/qwbmp.dll",
                    "./" + DISTRO_DIR + "/plugins/imageformats/qwebp.dll",
                    "./" + DISTRO_DIR + "/plugins/platforminputcontexts/qtvirtualkeyboardplugin.dll",
                    "./" + DISTRO_DIR + "/plugins/platforms/qminimal.dll",
                    "./" + DISTRO_DIR + "/plugins/platforms/qoffscreen.dll",
                    "./" + DISTRO_DIR + "/plugins/platforms/qwebgl.dll",
                    "./" + DISTRO_DIR + "/plugins/platforms/qwindows.dll",
                    "./" + DISTRO_DIR + "/plugins/virtualkeyboard/qtvirtualkeyboard_hangul.dll",
                    "./" + DISTRO_DIR + "/plugins/virtualkeyboard/qtvirtualkeyboard_openwnn.dll",
                    "./" + DISTRO_DIR + "/plugins/virtualkeyboard/qtvirtualkeyboard_pinyin.dll",
                    "./" + DISTRO_DIR + "/plugins/virtualkeyboard/qtvirtualkeyboard_tcime.dll",
                    "./" + DISTRO_DIR + "/plugins/virtualkeyboard/qtvirtualkeyboard_thai.dll",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/Blend.qml",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/BrightnessContrast.qml",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/ColorOverlay.qml",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/Colorize.qml",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/ConicalGradient.qml",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/Desaturate.qml",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/DirectionalBlur.qml",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/Displace.qml",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/DropShadow.qml",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/FastBlur.qml",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/GammaAdjust.qml",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/GaussianBlur.qml",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/Glow.qml",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/HueSaturation.qml",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/InnerShadow.qml",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/LevelAdjust.qml",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/LinearGradient.qml",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/MaskedBlur.qml",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/OpacityMask.qml",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/RadialBlur.qml",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/RadialGradient.qml",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/RectangularGlow.qml",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/RecursiveBlur.qml",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/ThresholdMask.qml",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/ZoomBlur.qml",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/plugins.qmltypes",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/private/DropShadowBase.qml",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/private/DropShadowBase.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/private/FastGlow.qml",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/private/FastGlow.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/private/FastInnerShadow.qml",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/private/FastInnerShadow.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/private/FastMaskedBlur.qml",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/private/FastMaskedBlur.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/private/GaussianDirectionalBlur.qml",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/private/GaussianDirectionalBlur.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/private/GaussianGlow.qml",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/private/GaussianGlow.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/private/GaussianInnerShadow.qml",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/private/GaussianInnerShadow.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/private/GaussianMaskedBlur.qml",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/private/GaussianMaskedBlur.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/private/qmldir",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/private/qtgraphicaleffectsprivate.dll",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/qmldir",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/qtgraphicaleffectsplugin.dll",
                    "./" + DISTRO_DIR + "/qml/QtQuick.2/plugins.qmltypes",
                    "./" + DISTRO_DIR + "/qml/QtQuick.2/qmldir",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/SplitView.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/SplitView.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/SplitView.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/SplitView.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/SplitView.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick.2/qtquick2plugin.dll",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/AbstractButton.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Action.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/ActionGroup.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/ApplicationWindow.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/BusyIndicator.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Button.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/ButtonGroup.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/CheckBox.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/CheckDelegate.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/ComboBox.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Container.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Control.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/DelayButton.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Dial.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Dialog.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/DialogButtonBox.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Drawer.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Frame.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/ApplicationWindow.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/BusyIndicator.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/Button.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/ButtonPanel.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/CheckBox.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/CheckDelegate.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/CheckIndicator.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/ComboBox.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/DelayButton.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/Dial.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/Dialog.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/DialogButtonBox.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/Drawer.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/Frame.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/GroupBox.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/ItemDelegate.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/Label.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/Menu.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/MenuBar.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/MenuBarItem.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/MenuItem.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/MenuSeparator.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/Page.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/PageIndicator.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/Pane.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/Popup.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/ProgressBar.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/RadioButton.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/RadioDelegate.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/RadioIndicator.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/RangeSlider.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/RoundButton.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/ScrollBar.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/ScrollIndicator.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/Slider.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/SliderGroove.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/SliderHandle.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/SpinBox.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/SwipeDelegate.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/Switch.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/SwitchDelegate.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/SwitchIndicator.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/TabBar.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/TabButton.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/TextArea.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/TextField.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/ToolBar.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/ToolButton.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/ToolSeparator.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/ToolTip.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/Tumbler.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/plugins.qmltypes",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/qmldir",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/qtquickcontrols2fusionstyleplugin.dll",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/GroupBox.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/ApplicationWindow.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/BusyIndicator.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/Button.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/CheckBox.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/CheckDelegate.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/ComboBox.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/DelayButton.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/Dial.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/Dialog.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/DialogButtonBox.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/Drawer.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/Frame.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/GroupBox.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/ItemDelegate.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/Label.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/Menu.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/MenuItem.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/MenuSeparator.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/Page.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/PageIndicator.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/Pane.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/Popup.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/ProgressBar.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/RadioButton.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/RadioDelegate.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/RangeSlider.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/RoundButton.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/ScrollBar.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/ScrollIndicator.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/Slider.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/SpinBox.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/StackView.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/SwipeDelegate.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/SwipeView.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/Switch.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/SwitchDelegate.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/TabBar.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/TabButton.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/TextArea.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/TextField.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/ToolBar.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/ToolButton.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/ToolSeparator.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/ToolTip.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/Tumbler.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/plugins.qmltypes",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/qmldir",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/qtquickcontrols2imaginestyleplugin.dll",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/ItemDelegate.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Label.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/ApplicationWindow.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/BoxShadow.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/BusyIndicator.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/Button.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/CheckBox.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/CheckDelegate.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/CheckIndicator.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/ComboBox.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/CursorDelegate.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/DelayButton.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/Dial.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/Dialog.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/DialogButtonBox.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/Drawer.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/ElevationEffect.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/Frame.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/GroupBox.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/ItemDelegate.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/Label.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/Menu.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/MenuBar.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/MenuBarItem.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/MenuItem.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/MenuSeparator.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/Page.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/PageIndicator.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/Pane.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/Popup.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/ProgressBar.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/RadioButton.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/RadioDelegate.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/RadioIndicator.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/RangeSlider.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/RectangularGlow.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/RoundButton.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/ScrollBar.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/ScrollIndicator.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/Slider.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/SliderHandle.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/SpinBox.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/StackView.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/SwipeDelegate.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/SwipeView.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/Switch.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/SwitchDelegate.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/SwitchIndicator.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/TabBar.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/TabButton.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/TextArea.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/TextField.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/ToolBar.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/ToolButton.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/ToolSeparator.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/ToolTip.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/Tumbler.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/plugins.qmltypes",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/qmldir",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/qtquickcontrols2materialstyleplugin.dll",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Menu.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/MenuBar.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/MenuBarItem.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/MenuItem.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/MenuSeparator.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Page.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/PageIndicator.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Pane.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Popup.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/ProgressBar.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/RadioButton.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/RadioDelegate.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/RangeSlider.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/RoundButton.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/ScrollBar.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/ScrollIndicator.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/ScrollView.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Slider.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/SpinBox.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/StackView.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/SwipeDelegate.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/SwipeView.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Switch.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/SwitchDelegate.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/TabBar.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/TabButton.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/TextArea.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/TextField.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/ToolBar.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/ToolButton.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/ToolSeparator.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/ToolTip.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Tumbler.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/ApplicationWindow.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/BusyIndicator.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/Button.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/CheckBox.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/CheckDelegate.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/CheckIndicator.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/ComboBox.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/DelayButton.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/Dial.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/Dialog.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/DialogButtonBox.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/Drawer.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/Frame.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/GroupBox.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/ItemDelegate.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/Label.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/Menu.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/MenuBar.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/MenuBarItem.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/MenuItem.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/MenuSeparator.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/Page.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/PageIndicator.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/Pane.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/Popup.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/ProgressBar.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/RadioButton.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/RadioDelegate.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/RadioIndicator.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/RangeSlider.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/RoundButton.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/ScrollBar.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/ScrollIndicator.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/Slider.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/SpinBox.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/StackView.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/SwipeDelegate.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/Switch.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/SwitchDelegate.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/SwitchIndicator.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/TabBar.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/TabButton.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/TextArea.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/TextField.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/ToolBar.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/ToolButton.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/ToolSeparator.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/ToolTip.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/Tumbler.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/plugins.qmltypes",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/qmldir",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/qtquickcontrols2universalstyleplugin.dll",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/AbstractButtonSection.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/BusyIndicatorSpecifics.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/ButtonSection.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/ButtonSpecifics.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/CheckBoxSpecifics.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/CheckDelegateSpecifics.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/CheckSection.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/ComboBoxSpecifics.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/ContainerSection.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/ControlSection.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/ControlSpecifics.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/DelayButtonSpecifics.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/DialSpecifics.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/FrameSpecifics.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/GroupBoxSpecifics.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/ItemDelegateSection.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/ItemDelegateSpecifics.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/LabelSpecifics.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/PaddingSection.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/PageIndicatorSpecifics.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/PageSpecifics.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/PaneSection.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/PaneSpecifics.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/ProgressBarSpecifics.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/RadioButtonSpecifics.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/RadioDelegateSpecifics.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/RangeSliderSpecifics.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/RoundButtonSpecifics.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/ScrollViewSpecifics.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/SliderSpecifics.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/SpinBoxSpecifics.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/StackViewSpecifics.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/SwipeDelegateSpecifics.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/SwipeViewSpecifics.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/SwitchDelegateSpecifics.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/SwitchSpecifics.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/TabBarSpecifics.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/TabButtonSpecifics.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/TextAreaSpecifics.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/TextFieldSpecifics.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/ToolBarSpecifics.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/ToolButtonSpecifics.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/ToolSeparatorSpecifics.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/TumblerSpecifics.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/busyindicator-icon.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/busyindicator-icon16.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/busyindicator-icon@2x.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/button-icon.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/button-icon16.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/button-icon@2x.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/checkbox-icon.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/checkbox-icon16.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/checkbox-icon@2x.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/combobox-icon.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/combobox-icon16.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/combobox-icon@2x.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/delaybutton-icon.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/delaybutton-icon16.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/delaybutton-icon@2x.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/dial-icon.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/dial-icon16.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/dial-icon@2x.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/frame-icon.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/frame-icon16.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/frame-icon@2x.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/groupbox-icon.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/groupbox-icon16.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/groupbox-icon@2x.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/itemdelegate-icon.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/itemdelegate-icon16.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/itemdelegate-icon@2x.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/label-icon.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/label-icon16.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/label-icon@2x.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/page-icon.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/page-icon16.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/page-icon@2x.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/pageindicator-icon.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/pageindicator-icon16.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/pageindicator-icon@2x.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/pane-icon.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/pane-icon16.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/pane-icon@2x.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/progressbar-icon.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/progressbar-icon16.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/progressbar-icon@2x.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/radiobutton-icon.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/radiobutton-icon16.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/radiobutton-icon@2x.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/rangeslider-icon.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/rangeslider-icon16.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/rangeslider-icon@2x.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/roundbutton-icon.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/roundbutton-icon16.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/roundbutton-icon@2x.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/scrollview-icon.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/scrollview-icon16.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/scrollview-icon@2x.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/slider-icon.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/slider-icon16.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/slider-icon@2x.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/spinbox-icon.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/spinbox-icon16.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/spinbox-icon@2x.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/stackview-icon.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/stackview-icon16.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/stackview-icon@2x.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/swipeview-icon.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/swipeview-icon16.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/swipeview-icon@2x.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/switch-icon.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/switch-icon16.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/switch-icon@2x.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/textarea-icon.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/textarea-icon16.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/textarea-icon@2x.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/textfield-icon.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/textfield-icon16.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/textfield-icon@2x.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/toolbar-icon.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/toolbar-icon16.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/toolbar-icon@2x.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/toolbutton-icon.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/toolbutton-icon16.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/toolbutton-icon@2x.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/toolseparator-icon.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/toolseparator-icon16.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/toolseparator-icon@2x.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/tumbler-icon.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/tumbler-icon16.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/tumbler-icon@2x.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/qtquickcontrols2.metainfo",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/plugins.qmltypes",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/qmldir",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/qtquickcontrols2plugin.dll",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Layouts/plugins.qmltypes",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Layouts/qmldir",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Layouts/qquicklayoutsplugin.dll",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Templates.2/plugins.qmltypes",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Templates.2/qmldir",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Templates.2/qtquicktemplates2plugin.dll",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Window.2/plugins.qmltypes",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Window.2/qmldir",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Window.2/windowplugin.dll",
                    "./" + DISTRO_DIR + "/qt.conf",
                    "./" + DISTRO_DIR + "/translations/qtbase_ar.qm",
                    "./" + DISTRO_DIR + "/translations/qtbase_bg.qm",
                    "./" + DISTRO_DIR + "/translations/qtbase_ca.qm",
                    "./" + DISTRO_DIR + "/translations/qtbase_cs.qm",
                    "./" + DISTRO_DIR + "/translations/qtbase_da.qm",
                    "./" + DISTRO_DIR + "/translations/qtbase_de.qm",
                    "./" + DISTRO_DIR + "/translations/qtbase_en.qm",
                    "./" + DISTRO_DIR + "/translations/qtbase_es.qm",
                    "./" + DISTRO_DIR + "/translations/qtbase_fi.qm",
                    "./" + DISTRO_DIR + "/translations/qtbase_fr.qm",
                    "./" + DISTRO_DIR + "/translations/qtbase_gd.qm",
                    "./" + DISTRO_DIR + "/translations/qtbase_he.qm",
                    "./" + DISTRO_DIR + "/translations/qtbase_hu.qm",
                    "./" + DISTRO_DIR + "/translations/qtbase_it.qm",
                    "./" + DISTRO_DIR + "/translations/qtbase_ja.qm",
                    "./" + DISTRO_DIR + "/translations/qtbase_ko.qm",
                    "./" + DISTRO_DIR + "/translations/qtbase_lv.qm",
                    "./" + DISTRO_DIR + "/translations/qtbase_pl.qm",
                    "./" + DISTRO_DIR + "/translations/qtbase_ru.qm",
                    "./" + DISTRO_DIR + "/translations/qtbase_sk.qm",
                    "./" + DISTRO_DIR + "/translations/qtbase_uk.qm",
                    "./" + DISTRO_DIR + "/translations/qtbase_zh_TW.qm",
                    "./" + DISTRO_DIR + "/translations/qtdeclarative_bg.qm",
                    "./" + DISTRO_DIR + "/translations/qtdeclarative_da.qm",
                    "./" + DISTRO_DIR + "/translations/qtdeclarative_de.qm",
                    "./" + DISTRO_DIR + "/translations/qtdeclarative_en.qm",
                    "./" + DISTRO_DIR + "/translations/qtdeclarative_es.qm",
                    "./" + DISTRO_DIR + "/translations/qtdeclarative_fi.qm",
                    "./" + DISTRO_DIR + "/translations/qtdeclarative_fr.qm",
                    "./" + DISTRO_DIR + "/translations/qtdeclarative_hu.qm",
                    "./" + DISTRO_DIR + "/translations/qtdeclarative_ja.qm",
                    "./" + DISTRO_DIR + "/translations/qtdeclarative_ko.qm",
                    "./" + DISTRO_DIR + "/translations/qtdeclarative_lv.qm",
                    "./" + DISTRO_DIR + "/translations/qtdeclarative_pl.qm",
                    "./" + DISTRO_DIR + "/translations/qtdeclarative_ru.qm",
                    "./" + DISTRO_DIR + "/translations/qtdeclarative_sk.qm",
                    "./" + DISTRO_DIR + "/translations/qtdeclarative_uk.qm"
                });
#else


    auto tree = utils.createTree(
    {
                    "./" + DISTRO_DIR + "/bin/qt.conf",
                    "./" + DISTRO_DIR + "/TestQMLWidgets.sh",
                    "./" + DISTRO_DIR + "/bin/TestQMLWidgets",
                    "./" + DISTRO_DIR + "/lib/libQt5Core.so",
                    "./" + DISTRO_DIR + "/lib/libQt5DBus.so",
                    "./" + DISTRO_DIR + "/lib/libQt5EglFSDeviceIntegration.so",
                    "./" + DISTRO_DIR + "/lib/libQt5Gui.so",
                    "./" + DISTRO_DIR + "/lib/libQt5Network.so",
                    "./" + DISTRO_DIR + "/lib/libQt5Qml.so",
                    "./" + DISTRO_DIR + "/lib/libQt5Quick.so",
                    "./" + DISTRO_DIR + "/lib/libQt5QuickControls2.so",
                    "./" + DISTRO_DIR + "/lib/libQt5QuickTemplates2.so",
                    "./" + DISTRO_DIR + "/lib/libQt5Svg.so",
                    "./" + DISTRO_DIR + "/lib/libQt5VirtualKeyboard.so",
                    "./" + DISTRO_DIR + "/lib/libQt5WaylandClient.so",
                    "./" + DISTRO_DIR + "/lib/libQt5WaylandCompositor.so",
                    "./" + DISTRO_DIR + "/lib/libQt5WebSockets.so",
                    "./" + DISTRO_DIR + "/lib/libQt5Widgets.so",
                    "./" + DISTRO_DIR + "/lib/libQt5XcbQpa.so",
                    "./" + DISTRO_DIR + "/lib/libicudata.so",
                    "./" + DISTRO_DIR + "/lib/libicui18n.so",
                    "./" + DISTRO_DIR + "/lib/libicuuc.so",
                    "./" + DISTRO_DIR + "/plugins/bearer/libqconnmanbearer.so",
                    "./" + DISTRO_DIR + "/plugins/bearer/libqgenericbearer.so",
                    "./" + DISTRO_DIR + "/plugins/bearer/libqnmbearer.so",
                    "./" + DISTRO_DIR + "/plugins/iconengines/libqsvgicon.so",
                    "./" + DISTRO_DIR + "/plugins/imageformats/libqgif.so",
                    "./" + DISTRO_DIR + "/plugins/imageformats/libqicns.so",
                    "./" + DISTRO_DIR + "/plugins/imageformats/libqico.so",
                    "./" + DISTRO_DIR + "/plugins/imageformats/libqjpeg.so",
                    "./" + DISTRO_DIR + "/plugins/imageformats/libqsvg.so",
                    "./" + DISTRO_DIR + "/plugins/imageformats/libqtga.so",
                    "./" + DISTRO_DIR + "/plugins/imageformats/libqtiff.so",
                    "./" + DISTRO_DIR + "/plugins/imageformats/libqwbmp.so",
                    "./" + DISTRO_DIR + "/plugins/imageformats/libqwebp.so",
                    "./" + DISTRO_DIR + "/plugins/platforminputcontexts/libcomposeplatforminputcontextplugin.so",
                    "./" + DISTRO_DIR + "/plugins/platforminputcontexts/libibusplatforminputcontextplugin.so",
                    "./" + DISTRO_DIR + "/plugins/platforminputcontexts/libqtvirtualkeyboardplugin.so",
                    "./" + DISTRO_DIR + "/plugins/platforms/libqeglfs.so",
                    "./" + DISTRO_DIR + "/plugins/platforms/libqlinuxfb.so",
                    "./" + DISTRO_DIR + "/plugins/platforms/libqminimal.so",
                    "./" + DISTRO_DIR + "/plugins/platforms/libqminimalegl.so",
                    "./" + DISTRO_DIR + "/plugins/platforms/libqoffscreen.so",
                    "./" + DISTRO_DIR + "/plugins/platforms/libqvnc.so",
                    "./" + DISTRO_DIR + "/plugins/platforms/libqwayland-egl.so",
                    "./" + DISTRO_DIR + "/plugins/platforms/libqwayland-generic.so",
                    "./" + DISTRO_DIR + "/plugins/platforms/libqwayland-xcomposite-egl.so",
                    "./" + DISTRO_DIR + "/plugins/platforms/libqwayland-xcomposite-glx.so",
                    "./" + DISTRO_DIR + "/plugins/platforms/libqwebgl.so",
                    "./" + DISTRO_DIR + "/plugins/platforms/libqxcb.so",
                    "./" + DISTRO_DIR + "/plugins/virtualkeyboard/libqtvirtualkeyboard_hangul.so",
                    "./" + DISTRO_DIR + "/plugins/virtualkeyboard/libqtvirtualkeyboard_openwnn.so",
                    "./" + DISTRO_DIR + "/plugins/virtualkeyboard/libqtvirtualkeyboard_pinyin.so",
                    "./" + DISTRO_DIR + "/plugins/virtualkeyboard/libqtvirtualkeyboard_tcime.so",
                    "./" + DISTRO_DIR + "/plugins/virtualkeyboard/libqtvirtualkeyboard_thai.so",
                    "./" + DISTRO_DIR + "/plugins/wayland-decoration-client/libbradient.so",
                    "./" + DISTRO_DIR + "/plugins/wayland-graphics-integration-client/libdmabuf-server.so",
                    "./" + DISTRO_DIR + "/plugins/wayland-graphics-integration-client/libdrm-egl-server.so",
                    "./" + DISTRO_DIR + "/plugins/wayland-graphics-integration-client/libqt-plugin-wayland-egl.so",
                    "./" + DISTRO_DIR + "/plugins/wayland-graphics-integration-client/libshm-emulation-server.so",
                    "./" + DISTRO_DIR + "/plugins/wayland-graphics-integration-client/libxcomposite-egl.so",
                    "./" + DISTRO_DIR + "/plugins/wayland-graphics-integration-client/libxcomposite-glx.so",
                    "./" + DISTRO_DIR + "/plugins/wayland-graphics-integration-server/libdmabuf-server.so",
                    "./" + DISTRO_DIR + "/plugins/wayland-graphics-integration-server/libdrm-egl-server.so",
                    "./" + DISTRO_DIR + "/plugins/wayland-graphics-integration-server/liblinux-dmabuf-unstable-v1.so",
                    "./" + DISTRO_DIR + "/plugins/wayland-graphics-integration-server/libqt-plugin-wayland-egl.so",
                    "./" + DISTRO_DIR + "/plugins/wayland-graphics-integration-server/libshm-emulation-server.so",
                    "./" + DISTRO_DIR + "/plugins/wayland-graphics-integration-server/libwayland-eglstream-controller.so",
                    "./" + DISTRO_DIR + "/plugins/wayland-graphics-integration-server/libxcomposite-egl.so",
                    "./" + DISTRO_DIR + "/plugins/wayland-graphics-integration-server/libxcomposite-glx.so",
                    "./" + DISTRO_DIR + "/plugins/wayland-shell-integration/libfullscreen-shell-v1.so",
                    "./" + DISTRO_DIR + "/plugins/wayland-shell-integration/libivi-shell.so",
                    "./" + DISTRO_DIR + "/plugins/wayland-shell-integration/libwl-shell.so",
                    "./" + DISTRO_DIR + "/plugins/wayland-shell-integration/libxdg-shell-v5.so",
                    "./" + DISTRO_DIR + "/plugins/wayland-shell-integration/libxdg-shell-v6.so",
                    "./" + DISTRO_DIR + "/plugins/wayland-shell-integration/libxdg-shell.so",
                    "./" + DISTRO_DIR + "/plugins/xcbglintegrations/libqxcb-egl-integration.so",
                    "./" + DISTRO_DIR + "/plugins/xcbglintegrations/libqxcb-glx-integration.so",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/Blend.qml",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/BrightnessContrast.qml",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/ColorOverlay.qml",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/Colorize.qml",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/ConicalGradient.qml",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/Desaturate.qml",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/DirectionalBlur.qml",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/Displace.qml",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/DropShadow.qml",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/FastBlur.qml",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/GammaAdjust.qml",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/GaussianBlur.qml",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/Glow.qml",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/HueSaturation.qml",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/InnerShadow.qml",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/LevelAdjust.qml",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/LinearGradient.qml",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/MaskedBlur.qml",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/OpacityMask.qml",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/RadialBlur.qml",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/RadialGradient.qml",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/RectangularGlow.qml",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/RecursiveBlur.qml",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/ThresholdMask.qml",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/ZoomBlur.qml",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/libqtgraphicaleffectsplugin.so",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/plugins.qmltypes",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/private/DropShadowBase.qml",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/private/DropShadowBase.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/private/FastGlow.qml",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/private/FastGlow.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/private/FastInnerShadow.qml",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/private/FastInnerShadow.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/private/FastMaskedBlur.qml",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/private/FastMaskedBlur.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/private/GaussianDirectionalBlur.qml",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/private/GaussianDirectionalBlur.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/private/GaussianGlow.qml",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/private/GaussianGlow.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/private/GaussianInnerShadow.qml",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/private/GaussianInnerShadow.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/private/GaussianMaskedBlur.qml",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/private/GaussianMaskedBlur.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/private/libqtgraphicaleffectsprivate.so",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/private/qmldir",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/qmldir",
                    "./" + DISTRO_DIR + "/qml/QtQuick.2/libqtquick2plugin.so",
                    "./" + DISTRO_DIR + "/qml/QtQuick.2/plugins.qmltypes",
                    "./" + DISTRO_DIR + "/qml/QtQuick.2/qmldir",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/AbstractButton.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Action.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/ActionGroup.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/ApplicationWindow.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/BusyIndicator.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Button.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/ButtonGroup.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/CheckBox.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/CheckDelegate.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/ComboBox.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Container.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Control.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/DelayButton.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Dial.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Dialog.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/DialogButtonBox.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Drawer.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Frame.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/ApplicationWindow.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/BusyIndicator.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/Button.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/ButtonPanel.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/CheckBox.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/CheckDelegate.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/CheckIndicator.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/ComboBox.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/DelayButton.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/Dial.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/Dialog.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/DialogButtonBox.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/Drawer.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/Frame.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/GroupBox.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/ItemDelegate.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/Label.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/Menu.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/MenuBar.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/MenuBarItem.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/MenuItem.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/MenuSeparator.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/Page.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/PageIndicator.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/Pane.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/Popup.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/ProgressBar.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/RadioButton.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/RadioDelegate.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/RadioIndicator.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/RangeSlider.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/RoundButton.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/ScrollBar.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/ScrollIndicator.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/Slider.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/SliderGroove.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/SliderHandle.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/SpinBox.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/SplitView.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/SwipeDelegate.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/Switch.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/SwitchDelegate.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/SwitchIndicator.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/TabBar.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/TabButton.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/TextArea.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/TextField.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/ToolBar.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/ToolButton.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/ToolSeparator.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/ToolTip.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/Tumbler.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/libqtquickcontrols2fusionstyleplugin.so",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/plugins.qmltypes",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/qmldir",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/GroupBox.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/ApplicationWindow.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/BusyIndicator.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/Button.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/CheckBox.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/CheckDelegate.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/ComboBox.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/DelayButton.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/Dial.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/Dialog.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/DialogButtonBox.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/Drawer.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/Frame.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/GroupBox.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/ItemDelegate.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/Label.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/Menu.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/MenuItem.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/MenuSeparator.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/Page.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/PageIndicator.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/Pane.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/Popup.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/ProgressBar.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/RadioButton.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/RadioDelegate.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/RangeSlider.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/RoundButton.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/ScrollBar.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/ScrollIndicator.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/Slider.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/SpinBox.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/SplitView.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/StackView.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/SwipeDelegate.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/SwipeView.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/Switch.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/SwitchDelegate.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/TabBar.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/TabButton.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/TextArea.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/TextField.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/ToolBar.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/ToolButton.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/ToolSeparator.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/ToolTip.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/Tumbler.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/libqtquickcontrols2imaginestyleplugin.so",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/plugins.qmltypes",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/qmldir",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/ItemDelegate.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Label.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/ApplicationWindow.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/BoxShadow.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/BusyIndicator.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/Button.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/CheckBox.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/CheckDelegate.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/CheckIndicator.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/ComboBox.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/CursorDelegate.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/DelayButton.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/Dial.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/Dialog.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/DialogButtonBox.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/Drawer.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/ElevationEffect.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/Frame.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/GroupBox.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/ItemDelegate.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/Label.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/Menu.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/MenuBar.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/MenuBarItem.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/MenuItem.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/MenuSeparator.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/Page.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/PageIndicator.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/Pane.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/Popup.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/ProgressBar.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/RadioButton.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/RadioDelegate.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/RadioIndicator.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/RangeSlider.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/RectangularGlow.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/RoundButton.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/ScrollBar.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/ScrollIndicator.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/Slider.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/SliderHandle.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/SpinBox.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/SplitView.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/StackView.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/SwipeDelegate.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/SwipeView.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/Switch.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/SwitchDelegate.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/SwitchIndicator.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/TabBar.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/TabButton.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/TextArea.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/TextField.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/ToolBar.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/ToolButton.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/ToolSeparator.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/ToolTip.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/Tumbler.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/libqtquickcontrols2materialstyleplugin.so",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/plugins.qmltypes",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/qmldir",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Menu.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/MenuBar.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/MenuBarItem.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/MenuItem.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/MenuSeparator.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Page.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/PageIndicator.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Pane.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Popup.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/ProgressBar.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/RadioButton.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/RadioDelegate.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/RangeSlider.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/RoundButton.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/ScrollBar.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/ScrollIndicator.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/ScrollView.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Slider.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/SpinBox.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/SplitView.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/StackView.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/SwipeDelegate.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/SwipeView.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Switch.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/SwitchDelegate.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/TabBar.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/TabButton.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/TextArea.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/TextField.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/ToolBar.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/ToolButton.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/ToolSeparator.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/ToolTip.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Tumbler.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/ApplicationWindow.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/BusyIndicator.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/Button.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/CheckBox.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/CheckDelegate.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/CheckIndicator.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/ComboBox.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/DelayButton.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/Dial.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/Dialog.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/DialogButtonBox.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/Drawer.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/Frame.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/GroupBox.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/ItemDelegate.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/Label.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/Menu.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/MenuBar.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/MenuBarItem.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/MenuItem.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/MenuSeparator.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/Page.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/PageIndicator.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/Pane.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/Popup.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/ProgressBar.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/RadioButton.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/RadioDelegate.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/RadioIndicator.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/RangeSlider.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/RoundButton.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/ScrollBar.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/ScrollIndicator.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/Slider.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/SpinBox.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/SplitView.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/StackView.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/SwipeDelegate.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/Switch.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/SwitchDelegate.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/SwitchIndicator.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/TabBar.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/TabButton.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/TextArea.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/TextField.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/ToolBar.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/ToolButton.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/ToolSeparator.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/ToolTip.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/Tumbler.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/libqtquickcontrols2universalstyleplugin.so",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/plugins.qmltypes",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/qmldir",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/AbstractButtonSection.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/BusyIndicatorSpecifics.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/ButtonSection.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/ButtonSpecifics.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/CheckBoxSpecifics.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/CheckDelegateSpecifics.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/CheckSection.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/ComboBoxSpecifics.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/ContainerSection.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/ControlSection.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/ControlSpecifics.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/DelayButtonSpecifics.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/DialSpecifics.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/FrameSpecifics.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/GroupBoxSpecifics.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/ItemDelegateSection.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/ItemDelegateSpecifics.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/LabelSpecifics.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/PaddingSection.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/PageIndicatorSpecifics.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/PageSpecifics.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/PaneSection.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/PaneSpecifics.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/ProgressBarSpecifics.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/RadioButtonSpecifics.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/RadioDelegateSpecifics.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/RangeSliderSpecifics.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/RoundButtonSpecifics.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/ScrollViewSpecifics.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/SliderSpecifics.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/SpinBoxSpecifics.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/StackViewSpecifics.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/SwipeDelegateSpecifics.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/SwipeViewSpecifics.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/SwitchDelegateSpecifics.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/SwitchSpecifics.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/TabBarSpecifics.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/TabButtonSpecifics.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/TextAreaSpecifics.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/TextFieldSpecifics.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/ToolBarSpecifics.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/ToolButtonSpecifics.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/ToolSeparatorSpecifics.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/TumblerSpecifics.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/busyindicator-icon.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/busyindicator-icon16.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/busyindicator-icon@2x.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/button-icon.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/button-icon16.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/button-icon@2x.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/checkbox-icon.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/checkbox-icon16.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/checkbox-icon@2x.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/combobox-icon.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/combobox-icon16.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/combobox-icon@2x.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/delaybutton-icon.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/delaybutton-icon16.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/delaybutton-icon@2x.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/dial-icon.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/dial-icon16.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/dial-icon@2x.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/frame-icon.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/frame-icon16.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/frame-icon@2x.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/groupbox-icon.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/groupbox-icon16.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/groupbox-icon@2x.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/itemdelegate-icon.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/itemdelegate-icon16.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/itemdelegate-icon@2x.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/label-icon.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/label-icon16.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/label-icon@2x.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/page-icon.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/page-icon16.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/page-icon@2x.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/pageindicator-icon.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/pageindicator-icon16.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/pageindicator-icon@2x.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/pane-icon.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/pane-icon16.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/pane-icon@2x.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/progressbar-icon.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/progressbar-icon16.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/progressbar-icon@2x.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/radiobutton-icon.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/radiobutton-icon16.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/radiobutton-icon@2x.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/rangeslider-icon.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/rangeslider-icon16.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/rangeslider-icon@2x.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/roundbutton-icon.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/roundbutton-icon16.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/roundbutton-icon@2x.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/scrollview-icon.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/scrollview-icon16.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/scrollview-icon@2x.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/slider-icon.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/slider-icon16.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/slider-icon@2x.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/spinbox-icon.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/spinbox-icon16.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/spinbox-icon@2x.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/stackview-icon.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/stackview-icon16.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/stackview-icon@2x.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/swipeview-icon.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/swipeview-icon16.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/swipeview-icon@2x.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/switch-icon.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/switch-icon16.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/switch-icon@2x.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/textarea-icon.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/textarea-icon16.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/textarea-icon@2x.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/textfield-icon.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/textfield-icon16.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/textfield-icon@2x.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/toolbar-icon.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/toolbar-icon16.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/toolbar-icon@2x.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/toolbutton-icon.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/toolbutton-icon16.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/toolbutton-icon@2x.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/toolseparator-icon.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/toolseparator-icon16.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/toolseparator-icon@2x.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/tumbler-icon.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/tumbler-icon16.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/tumbler-icon@2x.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/qtquickcontrols2.metainfo",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/libqtquickcontrols2plugin.so",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/plugins.qmltypes",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/qmldir",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Layouts/libqquicklayoutsplugin.so",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Layouts/plugins.qmltypes",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Layouts/qmldir",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Templates.2/libqtquicktemplates2plugin.so",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Templates.2/plugins.qmltypes",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Templates.2/qmldir",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Window.2/libwindowplugin.so",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Window.2/plugins.qmltypes",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Window.2/qmldir",
                    "./" + DISTRO_DIR + "/translations/qtbase_ar.qm",
                    "./" + DISTRO_DIR + "/translations/qtbase_bg.qm",
                    "./" + DISTRO_DIR + "/translations/qtbase_ca.qm",
                    "./" + DISTRO_DIR + "/translations/qtbase_cs.qm",
                    "./" + DISTRO_DIR + "/translations/qtbase_da.qm",
                    "./" + DISTRO_DIR + "/translations/qtbase_de.qm",
                    "./" + DISTRO_DIR + "/translations/qtbase_en.qm",
                    "./" + DISTRO_DIR + "/translations/qtbase_es.qm",
                    "./" + DISTRO_DIR + "/translations/qtbase_fi.qm",
                    "./" + DISTRO_DIR + "/translations/qtbase_fr.qm",
                    "./" + DISTRO_DIR + "/translations/qtbase_gd.qm",
                    "./" + DISTRO_DIR + "/translations/qtbase_he.qm",
                    "./" + DISTRO_DIR + "/translations/qtbase_hu.qm",
                    "./" + DISTRO_DIR + "/translations/qtbase_it.qm",
                    "./" + DISTRO_DIR + "/translations/qtbase_ja.qm",
                    "./" + DISTRO_DIR + "/translations/qtbase_ko.qm",
                    "./" + DISTRO_DIR + "/translations/qtbase_lv.qm",
                    "./" + DISTRO_DIR + "/translations/qtbase_pl.qm",
                    "./" + DISTRO_DIR + "/translations/qtbase_ru.qm",
                    "./" + DISTRO_DIR + "/translations/qtbase_sk.qm",
                    "./" + DISTRO_DIR + "/translations/qtbase_uk.qm",
                    "./" + DISTRO_DIR + "/translations/qtbase_zh_TW.qm",
                    "./" + DISTRO_DIR + "/translations/qtdeclarative_bg.qm",
                    "./" + DISTRO_DIR + "/translations/qtdeclarative_da.qm",
                    "./" + DISTRO_DIR + "/translations/qtdeclarative_de.qm",
                    "./" + DISTRO_DIR + "/translations/qtdeclarative_en.qm",
                    "./" + DISTRO_DIR + "/translations/qtdeclarative_es.qm",
                    "./" + DISTRO_DIR + "/translations/qtdeclarative_fi.qm",
                    "./" + DISTRO_DIR + "/translations/qtdeclarative_fr.qm",
                    "./" + DISTRO_DIR + "/translations/qtdeclarative_hu.qm",
                    "./" + DISTRO_DIR + "/translations/qtdeclarative_ja.qm",
                    "./" + DISTRO_DIR + "/translations/qtdeclarative_ko.qm",
                    "./" + DISTRO_DIR + "/translations/qtdeclarative_lv.qm",
                    "./" + DISTRO_DIR + "/translations/qtdeclarative_pl.qm",
                    "./" + DISTRO_DIR + "/translations/qtdeclarative_ru.qm",
                    "./" + DISTRO_DIR + "/translations/qtdeclarative_sk.qm",
                    "./" + DISTRO_DIR + "/translations/qtdeclarative_uk.qm"
                });
#endif

    return tree;
}

QSet<QString> Modules::outTestLibs() {
    TestUtils utils;

#ifdef Q_OS_WIN
    auto Tree = utils.createTree({
        "./" + DISTRO_DIR + "/TestQMLWidgets.bat",
        "./" + DISTRO_DIR + "/lol/TestQMLWidgets.exe",
        "./" + DISTRO_DIR + "/lol/qt.conf",
        "./" + DISTRO_DIR + "/lolLib/Qt5Core.dll",
        "./" + DISTRO_DIR + "/lolLib/Qt5Gui.dll",
        "./" + DISTRO_DIR + "/lolLib/Qt5Network.dll",
        "./" + DISTRO_DIR + "/lolLib/Qt5Qml.dll",
        "./" + DISTRO_DIR + "/lolLib/Qt5Quick.dll",
        "./" + DISTRO_DIR + "/lolLib/Qt5QuickControls2.dll",
        "./" + DISTRO_DIR + "/lolLib/Qt5QuickTemplates2.dll",
        "./" + DISTRO_DIR + "/lolLib/Qt5Svg.dll",
        "./" + DISTRO_DIR + "/lolLib/Qt5VirtualKeyboard.dll",
        "./" + DISTRO_DIR + "/lolLib/Qt5WebSockets.dll",
        "./" + DISTRO_DIR + "/lolLib/Qt5Widgets.dll",
        "./" + DISTRO_DIR + "/lolLib/libgcc_s_seh-1.dll",
        "./" + DISTRO_DIR + "/lolLib/libstdc++-6.dll",
        "./" + DISTRO_DIR + "/lolLib/libwinpthread-1.dll",
        "./" + DISTRO_DIR + "/lolTr/qtbase_ar.qm",
        "./" + DISTRO_DIR + "/lolTr/qtbase_bg.qm",
        "./" + DISTRO_DIR + "/lolTr/qtbase_ca.qm",
        "./" + DISTRO_DIR + "/lolTr/qtbase_cs.qm",
        "./" + DISTRO_DIR + "/lolTr/qtbase_da.qm",
        "./" + DISTRO_DIR + "/lolTr/qtbase_de.qm",
        "./" + DISTRO_DIR + "/lolTr/qtbase_en.qm",
        "./" + DISTRO_DIR + "/lolTr/qtbase_es.qm",
        "./" + DISTRO_DIR + "/lolTr/qtbase_fi.qm",
        "./" + DISTRO_DIR + "/lolTr/qtbase_fr.qm",
        "./" + DISTRO_DIR + "/lolTr/qtbase_gd.qm",
        "./" + DISTRO_DIR + "/lolTr/qtbase_he.qm",
        "./" + DISTRO_DIR + "/lolTr/qtbase_hu.qm",
        "./" + DISTRO_DIR + "/lolTr/qtbase_it.qm",
        "./" + DISTRO_DIR + "/lolTr/qtbase_ja.qm",
        "./" + DISTRO_DIR + "/lolTr/qtbase_ko.qm",
        "./" + DISTRO_DIR + "/lolTr/qtbase_lv.qm",
        "./" + DISTRO_DIR + "/lolTr/qtbase_pl.qm",
        "./" + DISTRO_DIR + "/lolTr/qtbase_ru.qm",
        "./" + DISTRO_DIR + "/lolTr/qtbase_sk.qm",
        "./" + DISTRO_DIR + "/lolTr/qtbase_uk.qm",
        "./" + DISTRO_DIR + "/lolTr/qtbase_zh_TW.qm",
        "./" + DISTRO_DIR + "/lolTr/qtdeclarative_bg.qm",
        "./" + DISTRO_DIR + "/lolTr/qtdeclarative_da.qm",
        "./" + DISTRO_DIR + "/lolTr/qtdeclarative_de.qm",
        "./" + DISTRO_DIR + "/lolTr/qtdeclarative_en.qm",
        "./" + DISTRO_DIR + "/lolTr/qtdeclarative_es.qm",
        "./" + DISTRO_DIR + "/lolTr/qtdeclarative_fi.qm",
        "./" + DISTRO_DIR + "/lolTr/qtdeclarative_fr.qm",
        "./" + DISTRO_DIR + "/lolTr/qtdeclarative_hu.qm",
        "./" + DISTRO_DIR + "/lolTr/qtdeclarative_ja.qm",
        "./" + DISTRO_DIR + "/lolTr/qtdeclarative_ko.qm",
        "./" + DISTRO_DIR + "/lolTr/qtdeclarative_lv.qm",
        "./" + DISTRO_DIR + "/lolTr/qtdeclarative_pl.qm",
        "./" + DISTRO_DIR + "/lolTr/qtdeclarative_ru.qm",
        "./" + DISTRO_DIR + "/lolTr/qtdeclarative_sk.qm",
        "./" + DISTRO_DIR + "/lolTr/qtdeclarative_uk.qm",
        "./" + DISTRO_DIR + "/p/bearer/qgenericbearer.dll",
        "./" + DISTRO_DIR + "/p/iconengines/qsvgicon.dll",
        "./" + DISTRO_DIR + "/p/imageformats/qgif.dll",
        "./" + DISTRO_DIR + "/p/imageformats/qicns.dll",
        "./" + DISTRO_DIR + "/p/imageformats/qico.dll",
        "./" + DISTRO_DIR + "/p/imageformats/qjpeg.dll",
        "./" + DISTRO_DIR + "/p/imageformats/qsvg.dll",
        "./" + DISTRO_DIR + "/p/imageformats/qtga.dll",
        "./" + DISTRO_DIR + "/p/imageformats/qtiff.dll",
        "./" + DISTRO_DIR + "/p/imageformats/qwbmp.dll",
        "./" + DISTRO_DIR + "/p/imageformats/qwebp.dll",
        "./" + DISTRO_DIR + "/p/platforminputcontexts/qtvirtualkeyboardplugin.dll",
        "./" + DISTRO_DIR + "/p/platforms/qminimal.dll",
        "./" + DISTRO_DIR + "/p/platforms/qoffscreen.dll",
        "./" + DISTRO_DIR + "/p/platforms/qwebgl.dll",
        "./" + DISTRO_DIR + "/p/platforms/qwindows.dll",
        "./" + DISTRO_DIR + "/p/virtualkeyboard/qtvirtualkeyboard_hangul.dll",
        "./" + DISTRO_DIR + "/p/virtualkeyboard/qtvirtualkeyboard_openwnn.dll",
        "./" + DISTRO_DIR + "/p/virtualkeyboard/qtvirtualkeyboard_pinyin.dll",
        "./" + DISTRO_DIR + "/p/virtualkeyboard/qtvirtualkeyboard_tcime.dll",
        "./" + DISTRO_DIR + "/p/virtualkeyboard/qtvirtualkeyboard_thai.dll",
        "./" + DISTRO_DIR + "/q/QtGraphicalEffects/Blend.qml",
        "./" + DISTRO_DIR + "/q/QtGraphicalEffects/BrightnessContrast.qml",
        "./" + DISTRO_DIR + "/q/QtGraphicalEffects/ColorOverlay.qml",
        "./" + DISTRO_DIR + "/q/QtGraphicalEffects/Colorize.qml",
        "./" + DISTRO_DIR + "/q/QtGraphicalEffects/ConicalGradient.qml",
        "./" + DISTRO_DIR + "/q/QtGraphicalEffects/Desaturate.qml",
        "./" + DISTRO_DIR + "/q/QtGraphicalEffects/DirectionalBlur.qml",
        "./" + DISTRO_DIR + "/q/QtGraphicalEffects/Displace.qml",
        "./" + DISTRO_DIR + "/q/QtGraphicalEffects/DropShadow.qml",
        "./" + DISTRO_DIR + "/q/QtGraphicalEffects/FastBlur.qml",
        "./" + DISTRO_DIR + "/q/QtGraphicalEffects/GammaAdjust.qml",
        "./" + DISTRO_DIR + "/q/QtGraphicalEffects/GaussianBlur.qml",
        "./" + DISTRO_DIR + "/q/QtGraphicalEffects/Glow.qml",
        "./" + DISTRO_DIR + "/q/QtGraphicalEffects/HueSaturation.qml",
        "./" + DISTRO_DIR + "/q/QtGraphicalEffects/InnerShadow.qml",
        "./" + DISTRO_DIR + "/q/QtGraphicalEffects/LevelAdjust.qml",
        "./" + DISTRO_DIR + "/q/QtGraphicalEffects/LinearGradient.qml",
        "./" + DISTRO_DIR + "/q/QtGraphicalEffects/MaskedBlur.qml",
        "./" + DISTRO_DIR + "/q/QtGraphicalEffects/OpacityMask.qml",
        "./" + DISTRO_DIR + "/q/QtGraphicalEffects/RadialBlur.qml",
        "./" + DISTRO_DIR + "/q/QtGraphicalEffects/RadialGradient.qml",
        "./" + DISTRO_DIR + "/q/QtGraphicalEffects/RectangularGlow.qml",
        "./" + DISTRO_DIR + "/q/QtGraphicalEffects/RecursiveBlur.qml",
        "./" + DISTRO_DIR + "/q/QtGraphicalEffects/ThresholdMask.qml",
        "./" + DISTRO_DIR + "/q/QtGraphicalEffects/ZoomBlur.qml",
        "./" + DISTRO_DIR + "/q/QtGraphicalEffects/plugins.qmltypes",
        "./" + DISTRO_DIR + "/q/QtGraphicalEffects/private/DropShadowBase.qml",
        "./" + DISTRO_DIR + "/q/QtGraphicalEffects/private/DropShadowBase.qmlc",
        "./" + DISTRO_DIR + "/q/QtGraphicalEffects/private/FastGlow.qml",
        "./" + DISTRO_DIR + "/q/QtGraphicalEffects/private/FastGlow.qmlc",
        "./" + DISTRO_DIR + "/q/QtGraphicalEffects/private/FastInnerShadow.qml",
        "./" + DISTRO_DIR + "/q/QtGraphicalEffects/private/FastInnerShadow.qmlc",
        "./" + DISTRO_DIR + "/q/QtGraphicalEffects/private/FastMaskedBlur.qml",
        "./" + DISTRO_DIR + "/q/QtGraphicalEffects/private/FastMaskedBlur.qmlc",
        "./" + DISTRO_DIR + "/q/QtGraphicalEffects/private/GaussianDirectionalBlur.qml",
        "./" + DISTRO_DIR + "/q/QtGraphicalEffects/private/GaussianDirectionalBlur.qmlc",
        "./" + DISTRO_DIR + "/q/QtGraphicalEffects/private/GaussianGlow.qml",
        "./" + DISTRO_DIR + "/q/QtGraphicalEffects/private/GaussianGlow.qmlc",
        "./" + DISTRO_DIR + "/q/QtGraphicalEffects/private/GaussianInnerShadow.qml",
        "./" + DISTRO_DIR + "/q/QtGraphicalEffects/private/GaussianInnerShadow.qmlc",
        "./" + DISTRO_DIR + "/q/QtGraphicalEffects/private/GaussianMaskedBlur.qml",
        "./" + DISTRO_DIR + "/q/QtGraphicalEffects/private/GaussianMaskedBlur.qmlc",
        "./" + DISTRO_DIR + "/q/QtGraphicalEffects/private/qmldir",
        "./" + DISTRO_DIR + "/q/QtGraphicalEffects/private/qtgraphicaleffectsprivate.dll",
        "./" + DISTRO_DIR + "/q/QtGraphicalEffects/qmldir",
        "./" + DISTRO_DIR + "/q/QtGraphicalEffects/qtgraphicaleffectsplugin.dll",
        "./" + DISTRO_DIR + "/q/QtQuick.2/plugins.qmltypes",
        "./" + DISTRO_DIR + "/q/QtQuick.2/qmldir",
        "./" + DISTRO_DIR + "/q/QtQuick.2/qtquick2plugin.dll",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Fusion/SplitView.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Imagine/SplitView.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Material/SplitView.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/SplitView.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Universal/SplitView.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/AbstractButton.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Action.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/ActionGroup.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/ApplicationWindow.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/BusyIndicator.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Button.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/ButtonGroup.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/CheckBox.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/CheckDelegate.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/ComboBox.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Container.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Control.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/DelayButton.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Dial.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Dialog.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/DialogButtonBox.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Drawer.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Frame.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Fusion/ApplicationWindow.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Fusion/BusyIndicator.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Fusion/Button.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Fusion/ButtonPanel.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Fusion/CheckBox.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Fusion/CheckDelegate.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Fusion/CheckIndicator.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Fusion/ComboBox.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Fusion/DelayButton.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Fusion/Dial.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Fusion/Dialog.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Fusion/DialogButtonBox.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Fusion/Drawer.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Fusion/Frame.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Fusion/GroupBox.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Fusion/ItemDelegate.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Fusion/Label.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Fusion/Menu.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Fusion/MenuBar.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Fusion/MenuBarItem.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Fusion/MenuItem.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Fusion/MenuSeparator.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Fusion/Page.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Fusion/PageIndicator.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Fusion/Pane.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Fusion/Popup.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Fusion/ProgressBar.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Fusion/RadioButton.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Fusion/RadioDelegate.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Fusion/RadioIndicator.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Fusion/RangeSlider.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Fusion/RoundButton.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Fusion/ScrollBar.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Fusion/ScrollIndicator.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Fusion/Slider.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Fusion/SliderGroove.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Fusion/SliderHandle.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Fusion/SpinBox.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Fusion/SwipeDelegate.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Fusion/Switch.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Fusion/SwitchDelegate.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Fusion/SwitchIndicator.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Fusion/TabBar.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Fusion/TabButton.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Fusion/TextArea.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Fusion/TextField.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Fusion/ToolBar.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Fusion/ToolButton.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Fusion/ToolSeparator.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Fusion/ToolTip.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Fusion/Tumbler.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Fusion/plugins.qmltypes",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Fusion/qmldir",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Fusion/qtquickcontrols2fusionstyleplugin.dll",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/GroupBox.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Imagine/ApplicationWindow.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Imagine/BusyIndicator.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Imagine/Button.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Imagine/CheckBox.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Imagine/CheckDelegate.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Imagine/ComboBox.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Imagine/DelayButton.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Imagine/Dial.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Imagine/Dialog.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Imagine/DialogButtonBox.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Imagine/Drawer.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Imagine/Frame.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Imagine/GroupBox.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Imagine/ItemDelegate.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Imagine/Label.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Imagine/Menu.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Imagine/MenuItem.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Imagine/MenuSeparator.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Imagine/Page.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Imagine/PageIndicator.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Imagine/Pane.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Imagine/Popup.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Imagine/ProgressBar.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Imagine/RadioButton.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Imagine/RadioDelegate.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Imagine/RangeSlider.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Imagine/RoundButton.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Imagine/ScrollBar.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Imagine/ScrollIndicator.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Imagine/Slider.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Imagine/SpinBox.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Imagine/StackView.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Imagine/SwipeDelegate.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Imagine/SwipeView.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Imagine/Switch.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Imagine/SwitchDelegate.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Imagine/TabBar.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Imagine/TabButton.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Imagine/TextArea.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Imagine/TextField.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Imagine/ToolBar.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Imagine/ToolButton.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Imagine/ToolSeparator.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Imagine/ToolTip.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Imagine/Tumbler.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Imagine/plugins.qmltypes",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Imagine/qmldir",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Imagine/qtquickcontrols2imaginestyleplugin.dll",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/ItemDelegate.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Label.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Material/ApplicationWindow.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Material/BoxShadow.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Material/BusyIndicator.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Material/Button.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Material/CheckBox.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Material/CheckDelegate.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Material/CheckIndicator.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Material/ComboBox.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Material/CursorDelegate.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Material/DelayButton.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Material/Dial.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Material/Dialog.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Material/DialogButtonBox.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Material/Drawer.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Material/ElevationEffect.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Material/Frame.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Material/GroupBox.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Material/ItemDelegate.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Material/Label.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Material/Menu.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Material/MenuBar.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Material/MenuBarItem.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Material/MenuItem.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Material/MenuSeparator.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Material/Page.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Material/PageIndicator.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Material/Pane.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Material/Popup.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Material/ProgressBar.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Material/RadioButton.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Material/RadioDelegate.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Material/RadioIndicator.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Material/RangeSlider.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Material/RectangularGlow.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Material/RoundButton.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Material/ScrollBar.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Material/ScrollIndicator.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Material/Slider.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Material/SliderHandle.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Material/SpinBox.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Material/StackView.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Material/SwipeDelegate.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Material/SwipeView.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Material/Switch.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Material/SwitchDelegate.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Material/SwitchIndicator.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Material/TabBar.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Material/TabButton.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Material/TextArea.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Material/TextField.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Material/ToolBar.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Material/ToolButton.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Material/ToolSeparator.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Material/ToolTip.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Material/Tumbler.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Material/plugins.qmltypes",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Material/qmldir",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Material/qtquickcontrols2materialstyleplugin.dll",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Menu.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/MenuBar.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/MenuBarItem.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/MenuItem.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/MenuSeparator.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Page.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/PageIndicator.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Pane.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Popup.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/ProgressBar.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/RadioButton.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/RadioDelegate.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/RangeSlider.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/RoundButton.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/ScrollBar.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/ScrollIndicator.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/ScrollView.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Slider.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/SpinBox.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/StackView.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/SwipeDelegate.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/SwipeView.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Switch.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/SwitchDelegate.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/TabBar.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/TabButton.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/TextArea.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/TextField.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/ToolBar.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/ToolButton.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/ToolSeparator.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/ToolTip.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Tumbler.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Universal/ApplicationWindow.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Universal/BusyIndicator.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Universal/Button.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Universal/CheckBox.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Universal/CheckDelegate.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Universal/CheckIndicator.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Universal/ComboBox.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Universal/DelayButton.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Universal/Dial.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Universal/Dialog.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Universal/DialogButtonBox.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Universal/Drawer.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Universal/Frame.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Universal/GroupBox.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Universal/ItemDelegate.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Universal/Label.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Universal/Menu.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Universal/MenuBar.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Universal/MenuBarItem.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Universal/MenuItem.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Universal/MenuSeparator.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Universal/Page.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Universal/PageIndicator.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Universal/Pane.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Universal/Popup.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Universal/ProgressBar.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Universal/RadioButton.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Universal/RadioDelegate.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Universal/RadioIndicator.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Universal/RangeSlider.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Universal/RoundButton.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Universal/ScrollBar.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Universal/ScrollIndicator.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Universal/Slider.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Universal/SpinBox.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Universal/StackView.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Universal/SwipeDelegate.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Universal/Switch.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Universal/SwitchDelegate.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Universal/SwitchIndicator.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Universal/TabBar.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Universal/TabButton.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Universal/TextArea.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Universal/TextField.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Universal/ToolBar.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Universal/ToolButton.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Universal/ToolSeparator.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Universal/ToolTip.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Universal/Tumbler.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Universal/plugins.qmltypes",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Universal/qmldir",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Universal/qtquickcontrols2universalstyleplugin.dll",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/AbstractButtonSection.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/BusyIndicatorSpecifics.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/ButtonSection.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/ButtonSpecifics.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/CheckBoxSpecifics.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/CheckDelegateSpecifics.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/CheckSection.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/ComboBoxSpecifics.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/ContainerSection.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/ControlSection.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/ControlSpecifics.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/DelayButtonSpecifics.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/DialSpecifics.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/FrameSpecifics.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/GroupBoxSpecifics.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/ItemDelegateSection.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/ItemDelegateSpecifics.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/LabelSpecifics.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/PaddingSection.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/PageIndicatorSpecifics.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/PageSpecifics.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/PaneSection.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/PaneSpecifics.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/ProgressBarSpecifics.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/RadioButtonSpecifics.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/RadioDelegateSpecifics.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/RangeSliderSpecifics.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/RoundButtonSpecifics.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/ScrollViewSpecifics.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/SliderSpecifics.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/SpinBoxSpecifics.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/StackViewSpecifics.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/SwipeDelegateSpecifics.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/SwipeViewSpecifics.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/SwitchDelegateSpecifics.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/SwitchSpecifics.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/TabBarSpecifics.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/TabButtonSpecifics.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/TextAreaSpecifics.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/TextFieldSpecifics.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/ToolBarSpecifics.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/ToolButtonSpecifics.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/ToolSeparatorSpecifics.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/TumblerSpecifics.qml",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/busyindicator-icon.png",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/busyindicator-icon16.png",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/busyindicator-icon@2x.png",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/button-icon.png",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/button-icon16.png",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/button-icon@2x.png",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/checkbox-icon.png",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/checkbox-icon16.png",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/checkbox-icon@2x.png",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/combobox-icon.png",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/combobox-icon16.png",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/combobox-icon@2x.png",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/delaybutton-icon.png",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/delaybutton-icon16.png",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/delaybutton-icon@2x.png",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/dial-icon.png",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/dial-icon16.png",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/dial-icon@2x.png",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/frame-icon.png",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/frame-icon16.png",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/frame-icon@2x.png",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/groupbox-icon.png",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/groupbox-icon16.png",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/groupbox-icon@2x.png",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/itemdelegate-icon.png",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/itemdelegate-icon16.png",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/itemdelegate-icon@2x.png",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/label-icon.png",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/label-icon16.png",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/label-icon@2x.png",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/page-icon.png",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/page-icon16.png",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/page-icon@2x.png",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/pageindicator-icon.png",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/pageindicator-icon16.png",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/pageindicator-icon@2x.png",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/pane-icon.png",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/pane-icon16.png",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/pane-icon@2x.png",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/progressbar-icon.png",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/progressbar-icon16.png",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/progressbar-icon@2x.png",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/radiobutton-icon.png",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/radiobutton-icon16.png",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/radiobutton-icon@2x.png",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/rangeslider-icon.png",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/rangeslider-icon16.png",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/rangeslider-icon@2x.png",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/roundbutton-icon.png",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/roundbutton-icon16.png",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/roundbutton-icon@2x.png",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/scrollview-icon.png",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/scrollview-icon16.png",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/scrollview-icon@2x.png",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/slider-icon.png",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/slider-icon16.png",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/slider-icon@2x.png",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/spinbox-icon.png",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/spinbox-icon16.png",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/spinbox-icon@2x.png",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/stackview-icon.png",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/stackview-icon16.png",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/stackview-icon@2x.png",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/swipeview-icon.png",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/swipeview-icon16.png",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/swipeview-icon@2x.png",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/switch-icon.png",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/switch-icon16.png",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/switch-icon@2x.png",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/textarea-icon.png",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/textarea-icon16.png",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/textarea-icon@2x.png",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/textfield-icon.png",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/textfield-icon16.png",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/textfield-icon@2x.png",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/toolbar-icon.png",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/toolbar-icon16.png",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/toolbar-icon@2x.png",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/toolbutton-icon.png",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/toolbutton-icon16.png",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/toolbutton-icon@2x.png",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/toolseparator-icon.png",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/toolseparator-icon16.png",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/toolseparator-icon@2x.png",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/tumbler-icon.png",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/tumbler-icon16.png",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/tumbler-icon@2x.png",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/qtquickcontrols2.metainfo",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/plugins.qmltypes",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/qmldir",
        "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/qtquickcontrols2plugin.dll",
        "./" + DISTRO_DIR + "/q/QtQuick/Layouts/plugins.qmltypes",
        "./" + DISTRO_DIR + "/q/QtQuick/Layouts/qmldir",
        "./" + DISTRO_DIR + "/q/QtQuick/Layouts/qquicklayoutsplugin.dll",
        "./" + DISTRO_DIR + "/q/QtQuick/Templates.2/plugins.qmltypes",
        "./" + DISTRO_DIR + "/q/QtQuick/Templates.2/qmldir",
        "./" + DISTRO_DIR + "/q/QtQuick/Templates.2/qtquicktemplates2plugin.dll",
        "./" + DISTRO_DIR + "/q/QtQuick/Window.2/plugins.qmltypes",
        "./" + DISTRO_DIR + "/q/QtQuick/Window.2/qmldir",
        "./" + DISTRO_DIR + "/q/QtQuick/Window.2/windowplugin.dll"
    });

#else
        auto Tree = utils.createTree({
             "./" + DISTRO_DIR + "/TestQMLWidgets.sh",
             "./" + DISTRO_DIR + "/lol/TestQMLWidgets",
             "./" + DISTRO_DIR + "/lol/qt.conf",
             "./" + DISTRO_DIR + "/lolLib/libQt5Core.so",
             "./" + DISTRO_DIR + "/lolLib/libQt5DBus.so",
             "./" + DISTRO_DIR + "/lolLib/libQt5EglFSDeviceIntegration.so",
             "./" + DISTRO_DIR + "/lolLib/libQt5Gui.so",
             "./" + DISTRO_DIR + "/lolLib/libQt5Network.so",
             "./" + DISTRO_DIR + "/lolLib/libQt5Qml.so",
             "./" + DISTRO_DIR + "/lolLib/libQt5Quick.so",
             "./" + DISTRO_DIR + "/lolLib/libQt5QuickControls2.so",
             "./" + DISTRO_DIR + "/lolLib/libQt5QuickTemplates2.so",
             "./" + DISTRO_DIR + "/lolLib/libQt5Svg.so",
             "./" + DISTRO_DIR + "/lolLib/libQt5VirtualKeyboard.so",
             "./" + DISTRO_DIR + "/lolLib/libQt5WaylandClient.so",
             "./" + DISTRO_DIR + "/lolLib/libQt5WaylandCompositor.so",
             "./" + DISTRO_DIR + "/lolLib/libQt5WebSockets.so",
             "./" + DISTRO_DIR + "/lolLib/libQt5Widgets.so",
             "./" + DISTRO_DIR + "/lolLib/libQt5XcbQpa.so",
             "./" + DISTRO_DIR + "/lolLib/libicudata.so",
             "./" + DISTRO_DIR + "/lolLib/libicui18n.so",
             "./" + DISTRO_DIR + "/lolLib/libicuuc.so",
             "./" + DISTRO_DIR + "/lolTr/qtbase_ar.qm",
             "./" + DISTRO_DIR + "/lolTr/qtbase_bg.qm",
             "./" + DISTRO_DIR + "/lolTr/qtbase_ca.qm",
             "./" + DISTRO_DIR + "/lolTr/qtbase_cs.qm",
             "./" + DISTRO_DIR + "/lolTr/qtbase_da.qm",
             "./" + DISTRO_DIR + "/lolTr/qtbase_de.qm",
             "./" + DISTRO_DIR + "/lolTr/qtbase_en.qm",
             "./" + DISTRO_DIR + "/lolTr/qtbase_es.qm",
             "./" + DISTRO_DIR + "/lolTr/qtbase_fi.qm",
             "./" + DISTRO_DIR + "/lolTr/qtbase_fr.qm",
             "./" + DISTRO_DIR + "/lolTr/qtbase_gd.qm",
             "./" + DISTRO_DIR + "/lolTr/qtbase_he.qm",
             "./" + DISTRO_DIR + "/lolTr/qtbase_hu.qm",
             "./" + DISTRO_DIR + "/lolTr/qtbase_it.qm",
             "./" + DISTRO_DIR + "/lolTr/qtbase_ja.qm",
             "./" + DISTRO_DIR + "/lolTr/qtbase_ko.qm",
             "./" + DISTRO_DIR + "/lolTr/qtbase_lv.qm",
             "./" + DISTRO_DIR + "/lolTr/qtbase_pl.qm",
             "./" + DISTRO_DIR + "/lolTr/qtbase_ru.qm",
             "./" + DISTRO_DIR + "/lolTr/qtbase_sk.qm",
             "./" + DISTRO_DIR + "/lolTr/qtbase_uk.qm",
             "./" + DISTRO_DIR + "/lolTr/qtbase_zh_TW.qm",
             "./" + DISTRO_DIR + "/lolTr/qtdeclarative_bg.qm",
             "./" + DISTRO_DIR + "/lolTr/qtdeclarative_da.qm",
             "./" + DISTRO_DIR + "/lolTr/qtdeclarative_de.qm",
             "./" + DISTRO_DIR + "/lolTr/qtdeclarative_en.qm",
             "./" + DISTRO_DIR + "/lolTr/qtdeclarative_es.qm",
             "./" + DISTRO_DIR + "/lolTr/qtdeclarative_fi.qm",
             "./" + DISTRO_DIR + "/lolTr/qtdeclarative_fr.qm",
             "./" + DISTRO_DIR + "/lolTr/qtdeclarative_hu.qm",
             "./" + DISTRO_DIR + "/lolTr/qtdeclarative_ja.qm",
             "./" + DISTRO_DIR + "/lolTr/qtdeclarative_ko.qm",
             "./" + DISTRO_DIR + "/lolTr/qtdeclarative_lv.qm",
             "./" + DISTRO_DIR + "/lolTr/qtdeclarative_pl.qm",
             "./" + DISTRO_DIR + "/lolTr/qtdeclarative_ru.qm",
             "./" + DISTRO_DIR + "/lolTr/qtdeclarative_sk.qm",
             "./" + DISTRO_DIR + "/lolTr/qtdeclarative_uk.qm",
             "./" + DISTRO_DIR + "/p/bearer/libqconnmanbearer.so",
             "./" + DISTRO_DIR + "/p/bearer/libqgenericbearer.so",
             "./" + DISTRO_DIR + "/p/bearer/libqnmbearer.so",
             "./" + DISTRO_DIR + "/p/iconengines/libqsvgicon.so",
             "./" + DISTRO_DIR + "/p/imageformats/libqgif.so",
             "./" + DISTRO_DIR + "/p/imageformats/libqicns.so",
             "./" + DISTRO_DIR + "/p/imageformats/libqico.so",
             "./" + DISTRO_DIR + "/p/imageformats/libqjpeg.so",
             "./" + DISTRO_DIR + "/p/imageformats/libqsvg.so",
             "./" + DISTRO_DIR + "/p/imageformats/libqtga.so",
             "./" + DISTRO_DIR + "/p/imageformats/libqtiff.so",
             "./" + DISTRO_DIR + "/p/imageformats/libqwbmp.so",
             "./" + DISTRO_DIR + "/p/imageformats/libqwebp.so",
             "./" + DISTRO_DIR + "/p/platforminputcontexts/libcomposeplatforminputcontextplugin.so",
             "./" + DISTRO_DIR + "/p/platforminputcontexts/libibusplatforminputcontextplugin.so",
             "./" + DISTRO_DIR + "/p/platforminputcontexts/libqtvirtualkeyboardplugin.so",
             "./" + DISTRO_DIR + "/p/platforms/libqeglfs.so",
             "./" + DISTRO_DIR + "/p/platforms/libqlinuxfb.so",
             "./" + DISTRO_DIR + "/p/platforms/libqminimal.so",
             "./" + DISTRO_DIR + "/p/platforms/libqminimalegl.so",
             "./" + DISTRO_DIR + "/p/platforms/libqoffscreen.so",
             "./" + DISTRO_DIR + "/p/platforms/libqvnc.so",
             "./" + DISTRO_DIR + "/p/platforms/libqwayland-egl.so",
             "./" + DISTRO_DIR + "/p/platforms/libqwayland-generic.so",
             "./" + DISTRO_DIR + "/p/platforms/libqwayland-xcomposite-egl.so",
             "./" + DISTRO_DIR + "/p/platforms/libqwayland-xcomposite-glx.so",
             "./" + DISTRO_DIR + "/p/platforms/libqwebgl.so",
             "./" + DISTRO_DIR + "/p/platforms/libqxcb.so",
             "./" + DISTRO_DIR + "/p/virtualkeyboard/libqtvirtualkeyboard_hangul.so",
             "./" + DISTRO_DIR + "/p/virtualkeyboard/libqtvirtualkeyboard_openwnn.so",
             "./" + DISTRO_DIR + "/p/virtualkeyboard/libqtvirtualkeyboard_pinyin.so",
             "./" + DISTRO_DIR + "/p/virtualkeyboard/libqtvirtualkeyboard_tcime.so",
             "./" + DISTRO_DIR + "/p/virtualkeyboard/libqtvirtualkeyboard_thai.so",
             "./" + DISTRO_DIR + "/p/wayland-decoration-client/libbradient.so",
             "./" + DISTRO_DIR + "/p/wayland-graphics-integration-client/libdmabuf-server.so",
             "./" + DISTRO_DIR + "/p/wayland-graphics-integration-client/libdrm-egl-server.so",
             "./" + DISTRO_DIR + "/p/wayland-graphics-integration-client/libqt-plugin-wayland-egl.so",
             "./" + DISTRO_DIR + "/p/wayland-graphics-integration-client/libshm-emulation-server.so",
             "./" + DISTRO_DIR + "/p/wayland-graphics-integration-client/libxcomposite-egl.so",
             "./" + DISTRO_DIR + "/p/wayland-graphics-integration-client/libxcomposite-glx.so",
             "./" + DISTRO_DIR + "/p/wayland-graphics-integration-server/libdmabuf-server.so",
             "./" + DISTRO_DIR + "/p/wayland-graphics-integration-server/libdrm-egl-server.so",
             "./" + DISTRO_DIR + "/p/wayland-graphics-integration-server/liblinux-dmabuf-unstable-v1.so",
             "./" + DISTRO_DIR + "/p/wayland-graphics-integration-server/libqt-plugin-wayland-egl.so",
             "./" + DISTRO_DIR + "/p/wayland-graphics-integration-server/libshm-emulation-server.so",
             "./" + DISTRO_DIR + "/p/wayland-graphics-integration-server/libwayland-eglstream-controller.so",
             "./" + DISTRO_DIR + "/p/wayland-graphics-integration-server/libxcomposite-egl.so",
             "./" + DISTRO_DIR + "/p/wayland-graphics-integration-server/libxcomposite-glx.so",
             "./" + DISTRO_DIR + "/p/wayland-shell-integration/libfullscreen-shell-v1.so",
             "./" + DISTRO_DIR + "/p/wayland-shell-integration/libivi-shell.so",
             "./" + DISTRO_DIR + "/p/wayland-shell-integration/libwl-shell.so",
             "./" + DISTRO_DIR + "/p/wayland-shell-integration/libxdg-shell-v5.so",
             "./" + DISTRO_DIR + "/p/wayland-shell-integration/libxdg-shell-v6.so",
             "./" + DISTRO_DIR + "/p/wayland-shell-integration/libxdg-shell.so",
             "./" + DISTRO_DIR + "/p/xcbglintegrations/libqxcb-egl-integration.so",
             "./" + DISTRO_DIR + "/p/xcbglintegrations/libqxcb-glx-integration.so",
             "./" + DISTRO_DIR + "/q/QtGraphicalEffects/Blend.qml",
             "./" + DISTRO_DIR + "/q/QtGraphicalEffects/BrightnessContrast.qml",
             "./" + DISTRO_DIR + "/q/QtGraphicalEffects/ColorOverlay.qml",
             "./" + DISTRO_DIR + "/q/QtGraphicalEffects/Colorize.qml",
             "./" + DISTRO_DIR + "/q/QtGraphicalEffects/ConicalGradient.qml",
             "./" + DISTRO_DIR + "/q/QtGraphicalEffects/Desaturate.qml",
             "./" + DISTRO_DIR + "/q/QtGraphicalEffects/DirectionalBlur.qml",
             "./" + DISTRO_DIR + "/q/QtGraphicalEffects/Displace.qml",
             "./" + DISTRO_DIR + "/q/QtGraphicalEffects/DropShadow.qml",
             "./" + DISTRO_DIR + "/q/QtGraphicalEffects/FastBlur.qml",
             "./" + DISTRO_DIR + "/q/QtGraphicalEffects/GammaAdjust.qml",
             "./" + DISTRO_DIR + "/q/QtGraphicalEffects/GaussianBlur.qml",
             "./" + DISTRO_DIR + "/q/QtGraphicalEffects/Glow.qml",
             "./" + DISTRO_DIR + "/q/QtGraphicalEffects/HueSaturation.qml",
             "./" + DISTRO_DIR + "/q/QtGraphicalEffects/InnerShadow.qml",
             "./" + DISTRO_DIR + "/q/QtGraphicalEffects/LevelAdjust.qml",
             "./" + DISTRO_DIR + "/q/QtGraphicalEffects/LinearGradient.qml",
             "./" + DISTRO_DIR + "/q/QtGraphicalEffects/MaskedBlur.qml",
             "./" + DISTRO_DIR + "/q/QtGraphicalEffects/OpacityMask.qml",
             "./" + DISTRO_DIR + "/q/QtGraphicalEffects/RadialBlur.qml",
             "./" + DISTRO_DIR + "/q/QtGraphicalEffects/RadialGradient.qml",
             "./" + DISTRO_DIR + "/q/QtGraphicalEffects/RectangularGlow.qml",
             "./" + DISTRO_DIR + "/q/QtGraphicalEffects/RecursiveBlur.qml",
             "./" + DISTRO_DIR + "/q/QtGraphicalEffects/ThresholdMask.qml",
             "./" + DISTRO_DIR + "/q/QtGraphicalEffects/ZoomBlur.qml",
             "./" + DISTRO_DIR + "/q/QtGraphicalEffects/libqtgraphicaleffectsplugin.so",
             "./" + DISTRO_DIR + "/q/QtGraphicalEffects/plugins.qmltypes",
             "./" + DISTRO_DIR + "/q/QtGraphicalEffects/private/DropShadowBase.qml",
             "./" + DISTRO_DIR + "/q/QtGraphicalEffects/private/DropShadowBase.qmlc",
             "./" + DISTRO_DIR + "/q/QtGraphicalEffects/private/FastGlow.qml",
             "./" + DISTRO_DIR + "/q/QtGraphicalEffects/private/FastGlow.qmlc",
             "./" + DISTRO_DIR + "/q/QtGraphicalEffects/private/FastInnerShadow.qml",
             "./" + DISTRO_DIR + "/q/QtGraphicalEffects/private/FastInnerShadow.qmlc",
             "./" + DISTRO_DIR + "/q/QtGraphicalEffects/private/FastMaskedBlur.qml",
             "./" + DISTRO_DIR + "/q/QtGraphicalEffects/private/FastMaskedBlur.qmlc",
             "./" + DISTRO_DIR + "/q/QtGraphicalEffects/private/GaussianDirectionalBlur.qml",
             "./" + DISTRO_DIR + "/q/QtGraphicalEffects/private/GaussianDirectionalBlur.qmlc",
             "./" + DISTRO_DIR + "/q/QtGraphicalEffects/private/GaussianGlow.qml",
             "./" + DISTRO_DIR + "/q/QtGraphicalEffects/private/GaussianGlow.qmlc",
             "./" + DISTRO_DIR + "/q/QtGraphicalEffects/private/GaussianInnerShadow.qml",
             "./" + DISTRO_DIR + "/q/QtGraphicalEffects/private/GaussianInnerShadow.qmlc",
             "./" + DISTRO_DIR + "/q/QtGraphicalEffects/private/GaussianMaskedBlur.qml",
             "./" + DISTRO_DIR + "/q/QtGraphicalEffects/private/GaussianMaskedBlur.qmlc",
             "./" + DISTRO_DIR + "/q/QtGraphicalEffects/private/libqtgraphicaleffectsprivate.so",
             "./" + DISTRO_DIR + "/q/QtGraphicalEffects/private/qmldir",
             "./" + DISTRO_DIR + "/q/QtGraphicalEffects/qmldir",
             "./" + DISTRO_DIR + "/q/QtQuick.2/libqtquick2plugin.so",
             "./" + DISTRO_DIR + "/q/QtQuick.2/plugins.qmltypes",
             "./" + DISTRO_DIR + "/q/QtQuick.2/qmldir",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/AbstractButton.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Action.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/ActionGroup.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/ApplicationWindow.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/BusyIndicator.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Button.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/ButtonGroup.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/CheckBox.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/CheckDelegate.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/ComboBox.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Container.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Control.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/DelayButton.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Dial.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Dialog.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/DialogButtonBox.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Drawer.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Frame.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Fusion/ApplicationWindow.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Fusion/BusyIndicator.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Fusion/Button.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Fusion/ButtonPanel.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Fusion/CheckBox.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Fusion/CheckDelegate.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Fusion/CheckIndicator.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Fusion/ComboBox.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Fusion/DelayButton.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Fusion/Dial.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Fusion/Dialog.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Fusion/DialogButtonBox.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Fusion/Drawer.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Fusion/Frame.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Fusion/GroupBox.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Fusion/ItemDelegate.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Fusion/Label.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Fusion/Menu.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Fusion/MenuBar.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Fusion/MenuBarItem.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Fusion/MenuItem.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Fusion/MenuSeparator.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Fusion/Page.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Fusion/PageIndicator.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Fusion/Pane.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Fusion/Popup.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Fusion/ProgressBar.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Fusion/RadioButton.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Fusion/RadioDelegate.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Fusion/RadioIndicator.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Fusion/RangeSlider.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Fusion/RoundButton.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Fusion/ScrollBar.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Fusion/ScrollIndicator.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Fusion/Slider.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Fusion/SliderGroove.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Fusion/SliderHandle.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Fusion/SpinBox.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Fusion/SplitView.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Fusion/SwipeDelegate.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Fusion/Switch.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Fusion/SwitchDelegate.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Fusion/SwitchIndicator.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Fusion/TabBar.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Fusion/TabButton.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Fusion/TextArea.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Fusion/TextField.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Fusion/ToolBar.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Fusion/ToolButton.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Fusion/ToolSeparator.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Fusion/ToolTip.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Fusion/Tumbler.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Fusion/libqtquickcontrols2fusionstyleplugin.so",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Fusion/plugins.qmltypes",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Fusion/qmldir",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/GroupBox.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Imagine/ApplicationWindow.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Imagine/BusyIndicator.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Imagine/Button.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Imagine/CheckBox.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Imagine/CheckDelegate.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Imagine/ComboBox.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Imagine/DelayButton.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Imagine/Dial.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Imagine/Dialog.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Imagine/DialogButtonBox.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Imagine/Drawer.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Imagine/Frame.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Imagine/GroupBox.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Imagine/ItemDelegate.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Imagine/Label.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Imagine/Menu.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Imagine/MenuItem.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Imagine/MenuSeparator.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Imagine/Page.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Imagine/PageIndicator.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Imagine/Pane.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Imagine/Popup.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Imagine/ProgressBar.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Imagine/RadioButton.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Imagine/RadioDelegate.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Imagine/RangeSlider.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Imagine/RoundButton.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Imagine/ScrollBar.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Imagine/ScrollIndicator.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Imagine/Slider.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Imagine/SpinBox.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Imagine/SplitView.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Imagine/StackView.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Imagine/SwipeDelegate.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Imagine/SwipeView.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Imagine/Switch.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Imagine/SwitchDelegate.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Imagine/TabBar.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Imagine/TabButton.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Imagine/TextArea.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Imagine/TextField.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Imagine/ToolBar.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Imagine/ToolButton.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Imagine/ToolSeparator.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Imagine/ToolTip.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Imagine/Tumbler.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Imagine/libqtquickcontrols2imaginestyleplugin.so",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Imagine/plugins.qmltypes",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Imagine/qmldir",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/ItemDelegate.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Label.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Material/ApplicationWindow.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Material/BoxShadow.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Material/BusyIndicator.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Material/Button.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Material/CheckBox.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Material/CheckDelegate.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Material/CheckIndicator.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Material/ComboBox.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Material/CursorDelegate.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Material/DelayButton.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Material/Dial.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Material/Dialog.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Material/DialogButtonBox.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Material/Drawer.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Material/ElevationEffect.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Material/Frame.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Material/GroupBox.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Material/ItemDelegate.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Material/Label.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Material/Menu.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Material/MenuBar.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Material/MenuBarItem.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Material/MenuItem.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Material/MenuSeparator.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Material/Page.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Material/PageIndicator.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Material/Pane.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Material/Popup.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Material/ProgressBar.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Material/RadioButton.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Material/RadioDelegate.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Material/RadioIndicator.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Material/RangeSlider.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Material/RectangularGlow.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Material/RoundButton.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Material/ScrollBar.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Material/ScrollIndicator.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Material/Slider.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Material/SliderHandle.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Material/SpinBox.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Material/SplitView.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Material/StackView.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Material/SwipeDelegate.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Material/SwipeView.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Material/Switch.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Material/SwitchDelegate.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Material/SwitchIndicator.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Material/TabBar.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Material/TabButton.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Material/TextArea.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Material/TextField.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Material/ToolBar.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Material/ToolButton.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Material/ToolSeparator.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Material/ToolTip.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Material/Tumbler.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Material/libqtquickcontrols2materialstyleplugin.so",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Material/plugins.qmltypes",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Material/qmldir",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Menu.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/MenuBar.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/MenuBarItem.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/MenuItem.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/MenuSeparator.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Page.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/PageIndicator.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Pane.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Popup.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/ProgressBar.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/RadioButton.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/RadioDelegate.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/RangeSlider.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/RoundButton.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/ScrollBar.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/ScrollIndicator.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/ScrollView.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Slider.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/SpinBox.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/SplitView.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/StackView.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/SwipeDelegate.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/SwipeView.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Switch.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/SwitchDelegate.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/TabBar.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/TabButton.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/TextArea.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/TextField.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/ToolBar.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/ToolButton.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/ToolSeparator.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/ToolTip.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Tumbler.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Universal/ApplicationWindow.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Universal/BusyIndicator.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Universal/Button.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Universal/CheckBox.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Universal/CheckDelegate.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Universal/CheckIndicator.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Universal/ComboBox.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Universal/DelayButton.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Universal/Dial.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Universal/Dialog.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Universal/DialogButtonBox.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Universal/Drawer.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Universal/Frame.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Universal/GroupBox.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Universal/ItemDelegate.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Universal/Label.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Universal/Menu.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Universal/MenuBar.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Universal/MenuBarItem.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Universal/MenuItem.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Universal/MenuSeparator.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Universal/Page.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Universal/PageIndicator.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Universal/Pane.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Universal/Popup.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Universal/ProgressBar.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Universal/RadioButton.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Universal/RadioDelegate.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Universal/RadioIndicator.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Universal/RangeSlider.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Universal/RoundButton.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Universal/ScrollBar.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Universal/ScrollIndicator.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Universal/Slider.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Universal/SpinBox.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Universal/SplitView.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Universal/StackView.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Universal/SwipeDelegate.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Universal/Switch.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Universal/SwitchDelegate.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Universal/SwitchIndicator.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Universal/TabBar.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Universal/TabButton.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Universal/TextArea.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Universal/TextField.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Universal/ToolBar.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Universal/ToolButton.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Universal/ToolSeparator.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Universal/ToolTip.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Universal/Tumbler.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Universal/libqtquickcontrols2universalstyleplugin.so",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Universal/plugins.qmltypes",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/Universal/qmldir",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/AbstractButtonSection.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/BusyIndicatorSpecifics.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/ButtonSection.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/ButtonSpecifics.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/CheckBoxSpecifics.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/CheckDelegateSpecifics.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/CheckSection.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/ComboBoxSpecifics.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/ContainerSection.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/ControlSection.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/ControlSpecifics.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/DelayButtonSpecifics.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/DialSpecifics.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/FrameSpecifics.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/GroupBoxSpecifics.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/ItemDelegateSection.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/ItemDelegateSpecifics.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/LabelSpecifics.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/PaddingSection.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/PageIndicatorSpecifics.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/PageSpecifics.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/PaneSection.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/PaneSpecifics.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/ProgressBarSpecifics.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/RadioButtonSpecifics.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/RadioDelegateSpecifics.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/RangeSliderSpecifics.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/RoundButtonSpecifics.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/ScrollViewSpecifics.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/SliderSpecifics.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/SpinBoxSpecifics.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/StackViewSpecifics.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/SwipeDelegateSpecifics.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/SwipeViewSpecifics.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/SwitchDelegateSpecifics.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/SwitchSpecifics.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/TabBarSpecifics.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/TabButtonSpecifics.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/TextAreaSpecifics.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/TextFieldSpecifics.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/ToolBarSpecifics.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/ToolButtonSpecifics.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/ToolSeparatorSpecifics.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/TumblerSpecifics.qml",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/busyindicator-icon.png",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/busyindicator-icon16.png",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/busyindicator-icon@2x.png",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/button-icon.png",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/button-icon16.png",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/button-icon@2x.png",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/checkbox-icon.png",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/checkbox-icon16.png",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/checkbox-icon@2x.png",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/combobox-icon.png",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/combobox-icon16.png",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/combobox-icon@2x.png",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/delaybutton-icon.png",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/delaybutton-icon16.png",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/delaybutton-icon@2x.png",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/dial-icon.png",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/dial-icon16.png",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/dial-icon@2x.png",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/frame-icon.png",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/frame-icon16.png",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/frame-icon@2x.png",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/groupbox-icon.png",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/groupbox-icon16.png",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/groupbox-icon@2x.png",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/itemdelegate-icon.png",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/itemdelegate-icon16.png",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/itemdelegate-icon@2x.png",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/label-icon.png",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/label-icon16.png",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/label-icon@2x.png",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/page-icon.png",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/page-icon16.png",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/page-icon@2x.png",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/pageindicator-icon.png",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/pageindicator-icon16.png",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/pageindicator-icon@2x.png",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/pane-icon.png",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/pane-icon16.png",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/pane-icon@2x.png",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/progressbar-icon.png",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/progressbar-icon16.png",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/progressbar-icon@2x.png",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/radiobutton-icon.png",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/radiobutton-icon16.png",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/radiobutton-icon@2x.png",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/rangeslider-icon.png",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/rangeslider-icon16.png",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/rangeslider-icon@2x.png",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/roundbutton-icon.png",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/roundbutton-icon16.png",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/roundbutton-icon@2x.png",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/scrollview-icon.png",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/scrollview-icon16.png",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/scrollview-icon@2x.png",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/slider-icon.png",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/slider-icon16.png",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/slider-icon@2x.png",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/spinbox-icon.png",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/spinbox-icon16.png",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/spinbox-icon@2x.png",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/stackview-icon.png",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/stackview-icon16.png",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/stackview-icon@2x.png",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/swipeview-icon.png",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/swipeview-icon16.png",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/swipeview-icon@2x.png",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/switch-icon.png",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/switch-icon16.png",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/switch-icon@2x.png",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/textarea-icon.png",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/textarea-icon16.png",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/textarea-icon@2x.png",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/textfield-icon.png",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/textfield-icon16.png",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/textfield-icon@2x.png",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/toolbar-icon.png",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/toolbar-icon16.png",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/toolbar-icon@2x.png",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/toolbutton-icon.png",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/toolbutton-icon16.png",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/toolbutton-icon@2x.png",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/toolseparator-icon.png",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/toolseparator-icon16.png",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/toolseparator-icon@2x.png",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/tumbler-icon.png",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/tumbler-icon16.png",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/images/tumbler-icon@2x.png",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/designer/qtquickcontrols2.metainfo",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/libqtquickcontrols2plugin.so",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/plugins.qmltypes",
             "./" + DISTRO_DIR + "/q/QtQuick/Controls.2/qmldir",
             "./" + DISTRO_DIR + "/q/QtQuick/Layouts/libqquicklayoutsplugin.so",
             "./" + DISTRO_DIR + "/q/QtQuick/Layouts/plugins.qmltypes",
             "./" + DISTRO_DIR + "/q/QtQuick/Layouts/qmldir",
             "./" + DISTRO_DIR + "/q/QtQuick/Templates.2/libqtquicktemplates2plugin.so",
             "./" + DISTRO_DIR + "/q/QtQuick/Templates.2/plugins.qmltypes",
             "./" + DISTRO_DIR + "/q/QtQuick/Templates.2/qmldir",
             "./" + DISTRO_DIR + "/q/QtQuick/Window.2/libwindowplugin.so",
             "./" + DISTRO_DIR + "/q/QtQuick/Window.2/plugins.qmltypes",
             "./" + DISTRO_DIR + "/q/QtQuick/Window.2/qmldir"
         }
         );
#endif
    return Tree;

}

QSet<QString> Modules::qtWithoutTr() {
    TestUtils utils;
#ifdef Q_OS_WIN
    auto Tree = utils.createTree(
    {
                    "./" + DISTRO_DIR + "/Qt5Core.dll",
                    "./" + DISTRO_DIR + "/Qt5Gui.dll",
                    "./" + DISTRO_DIR + "/Qt5Network.dll",
                    "./" + DISTRO_DIR + "/Qt5Qml.dll",
                    "./" + DISTRO_DIR + "/Qt5Quick.dll",
                    "./" + DISTRO_DIR + "/Qt5Svg.dll",
                    "./" + DISTRO_DIR + "/Qt5VirtualKeyboard.dll",
                    "./" + DISTRO_DIR + "/Qt5WebSockets.dll",
                    "./" + DISTRO_DIR + "/Qt5Widgets.dll",
                    "./" + DISTRO_DIR + "/QtWidgetsProject.exe",
                    "./" + DISTRO_DIR + "/libgcc_s_seh-1.dll",
                    "./" + DISTRO_DIR + "/libstdc++-6.dll",
                    "./" + DISTRO_DIR + "/libwinpthread-1.dll",
                    "./" + DISTRO_DIR + "/plugins/iconengines/qsvgicon.dll",
                    "./" + DISTRO_DIR + "/plugins/imageformats/qgif.dll",
                    "./" + DISTRO_DIR + "/plugins/imageformats/qicns.dll",
                    "./" + DISTRO_DIR + "/plugins/imageformats/qico.dll",
                    "./" + DISTRO_DIR + "/plugins/imageformats/qjpeg.dll",
                    "./" + DISTRO_DIR + "/plugins/imageformats/qsvg.dll",
                    "./" + DISTRO_DIR + "/plugins/imageformats/qtga.dll",
                    "./" + DISTRO_DIR + "/plugins/imageformats/qtiff.dll",
                    "./" + DISTRO_DIR + "/plugins/imageformats/qwbmp.dll",
                    "./" + DISTRO_DIR + "/plugins/imageformats/qwebp.dll",
                    "./" + DISTRO_DIR + "/plugins/platforminputcontexts/qtvirtualkeyboardplugin.dll",
                    "./" + DISTRO_DIR + "/plugins/platforms/qminimal.dll",
                    "./" + DISTRO_DIR + "/plugins/platforms/qoffscreen.dll",
                    "./" + DISTRO_DIR + "/plugins/platforms/qwebgl.dll",
                    "./" + DISTRO_DIR + "/plugins/platforms/qwindows.dll",
                    "./" + DISTRO_DIR + "/plugins/styles/qwindowsvistastyle.dll",
                    "./" + DISTRO_DIR + "/plugins/virtualkeyboard/qtvirtualkeyboard_hangul.dll",
                    "./" + DISTRO_DIR + "/plugins/virtualkeyboard/qtvirtualkeyboard_openwnn.dll",
                    "./" + DISTRO_DIR + "/plugins/virtualkeyboard/qtvirtualkeyboard_pinyin.dll",
                    "./" + DISTRO_DIR + "/plugins/virtualkeyboard/qtvirtualkeyboard_tcime.dll",
                    "./" + DISTRO_DIR + "/plugins/virtualkeyboard/qtvirtualkeyboard_thai.dll",
                    "./" + DISTRO_DIR + "/qt.conf"
                });
#else
    auto Tree = utils.createTree(
    {
                    "./" + DISTRO_DIR + "/bin/qt.conf",
                    "./" + DISTRO_DIR + "/QtWidgetsProject.sh",
                    "./" + DISTRO_DIR + "/bin/QtWidgetsProject",
                    "./" + DISTRO_DIR + "/lib/libQt5Core.so",
                    "./" + DISTRO_DIR + "/lib/libQt5DBus.so",
                    "./" + DISTRO_DIR + "/lib/libQt5EglFSDeviceIntegration.so",
                    "./" + DISTRO_DIR + "/lib/libQt5Gui.so",
                    "./" + DISTRO_DIR + "/lib/libQt5Network.so",
                    "./" + DISTRO_DIR + "/lib/libQt5Qml.so",
                    "./" + DISTRO_DIR + "/lib/libQt5Quick.so",
                    "./" + DISTRO_DIR + "/lib/libQt5Svg.so",
                    "./" + DISTRO_DIR + "/lib/libQt5VirtualKeyboard.so",
                    "./" + DISTRO_DIR + "/lib/libQt5WaylandClient.so",
                    "./" + DISTRO_DIR + "/lib/libQt5WaylandCompositor.so",
                    "./" + DISTRO_DIR + "/lib/libQt5WebSockets.so",
                    "./" + DISTRO_DIR + "/lib/libQt5Widgets.so",
                    "./" + DISTRO_DIR + "/lib/libQt5XcbQpa.so",
                    "./" + DISTRO_DIR + "/lib/libicudata.so",
                    "./" + DISTRO_DIR + "/lib/libicui18n.so",
                    "./" + DISTRO_DIR + "/lib/libicuuc.so",
                    "./" + DISTRO_DIR + "/plugins/iconengines/libqsvgicon.so",
                    "./" + DISTRO_DIR + "/plugins/imageformats/libqgif.so",
                    "./" + DISTRO_DIR + "/plugins/imageformats/libqicns.so",
                    "./" + DISTRO_DIR + "/plugins/imageformats/libqico.so",
                    "./" + DISTRO_DIR + "/plugins/imageformats/libqjpeg.so",
                    "./" + DISTRO_DIR + "/plugins/imageformats/libqsvg.so",
                    "./" + DISTRO_DIR + "/plugins/imageformats/libqtga.so",
                    "./" + DISTRO_DIR + "/plugins/imageformats/libqtiff.so",
                    "./" + DISTRO_DIR + "/plugins/imageformats/libqwbmp.so",
                    "./" + DISTRO_DIR + "/plugins/imageformats/libqwebp.so",
                    "./" + DISTRO_DIR + "/plugins/platforminputcontexts/libcomposeplatforminputcontextplugin.so",
                    "./" + DISTRO_DIR + "/plugins/platforminputcontexts/libibusplatforminputcontextplugin.so",
                    "./" + DISTRO_DIR + "/plugins/platforminputcontexts/libqtvirtualkeyboardplugin.so",
                    "./" + DISTRO_DIR + "/plugins/platforms/libqeglfs.so",
                    "./" + DISTRO_DIR + "/plugins/platforms/libqlinuxfb.so",
                    "./" + DISTRO_DIR + "/plugins/platforms/libqminimal.so",
                    "./" + DISTRO_DIR + "/plugins/platforms/libqminimalegl.so",
                    "./" + DISTRO_DIR + "/plugins/platforms/libqoffscreen.so",
                    "./" + DISTRO_DIR + "/plugins/platforms/libqvnc.so",
                    "./" + DISTRO_DIR + "/plugins/platforms/libqwayland-egl.so",
                    "./" + DISTRO_DIR + "/plugins/platforms/libqwayland-generic.so",
                    "./" + DISTRO_DIR + "/plugins/platforms/libqwayland-xcomposite-egl.so",
                    "./" + DISTRO_DIR + "/plugins/platforms/libqwayland-xcomposite-glx.so",
                    "./" + DISTRO_DIR + "/plugins/platforms/libqwebgl.so",
                    "./" + DISTRO_DIR + "/plugins/platforms/libqxcb.so",
                    "./" + DISTRO_DIR + "/plugins/virtualkeyboard/libqtvirtualkeyboard_hangul.so",
                    "./" + DISTRO_DIR + "/plugins/virtualkeyboard/libqtvirtualkeyboard_openwnn.so",
                    "./" + DISTRO_DIR + "/plugins/virtualkeyboard/libqtvirtualkeyboard_pinyin.so",
                    "./" + DISTRO_DIR + "/plugins/virtualkeyboard/libqtvirtualkeyboard_tcime.so",
                    "./" + DISTRO_DIR + "/plugins/virtualkeyboard/libqtvirtualkeyboard_thai.so",
                    "./" + DISTRO_DIR + "/plugins/wayland-decoration-client/libbradient.so",
                    "./" + DISTRO_DIR + "/plugins/wayland-graphics-integration-client/libdmabuf-server.so",
                    "./" + DISTRO_DIR + "/plugins/wayland-graphics-integration-client/libdrm-egl-server.so",
                    "./" + DISTRO_DIR + "/plugins/wayland-graphics-integration-client/libqt-plugin-wayland-egl.so",
                    "./" + DISTRO_DIR + "/plugins/wayland-graphics-integration-client/libshm-emulation-server.so",
                    "./" + DISTRO_DIR + "/plugins/wayland-graphics-integration-client/libxcomposite-egl.so",
                    "./" + DISTRO_DIR + "/plugins/wayland-graphics-integration-client/libxcomposite-glx.so",
                    "./" + DISTRO_DIR + "/plugins/wayland-graphics-integration-server/libdmabuf-server.so",
                    "./" + DISTRO_DIR + "/plugins/wayland-graphics-integration-server/libdrm-egl-server.so",
                    "./" + DISTRO_DIR + "/plugins/wayland-graphics-integration-server/liblinux-dmabuf-unstable-v1.so",
                    "./" + DISTRO_DIR + "/plugins/wayland-graphics-integration-server/libqt-plugin-wayland-egl.so",
                    "./" + DISTRO_DIR + "/plugins/wayland-graphics-integration-server/libshm-emulation-server.so",
                    "./" + DISTRO_DIR + "/plugins/wayland-graphics-integration-server/libwayland-eglstream-controller.so",
                    "./" + DISTRO_DIR + "/plugins/wayland-graphics-integration-server/libxcomposite-egl.so",
                    "./" + DISTRO_DIR + "/plugins/wayland-graphics-integration-server/libxcomposite-glx.so",
                    "./" + DISTRO_DIR + "/plugins/wayland-shell-integration/libfullscreen-shell-v1.so",
                    "./" + DISTRO_DIR + "/plugins/wayland-shell-integration/libivi-shell.so",
                    "./" + DISTRO_DIR + "/plugins/wayland-shell-integration/libwl-shell.so",
                    "./" + DISTRO_DIR + "/plugins/wayland-shell-integration/libxdg-shell-v5.so",
                    "./" + DISTRO_DIR + "/plugins/wayland-shell-integration/libxdg-shell-v6.so",
                    "./" + DISTRO_DIR + "/plugins/wayland-shell-integration/libxdg-shell.so",
                    "./" + DISTRO_DIR + "/plugins/xcbglintegrations/libqxcb-egl-integration.so",
                    "./" + DISTRO_DIR + "/plugins/xcbglintegrations/libqxcb-glx-integration.so"
                }
                );
#endif
    return Tree;
}

QSet<QString> Modules::qtWebEngine() {
    TestUtils utils;

    auto Tree = utils.createTree(
    {
                    "./" + DISTRO_DIR + "/bin/QtWebEngineProcess",
                    "./" + DISTRO_DIR + "/bin/qt.conf",
                    "./" + DISTRO_DIR + "/bin/quicknanobrowser",
                    "./" + DISTRO_DIR + "/lib/libQt5Core.so",
                    "./" + DISTRO_DIR + "/lib/libQt5DBus.so",
                    "./" + DISTRO_DIR + "/lib/libQt5EglFSDeviceIntegration.so",
                    "./" + DISTRO_DIR + "/lib/libQt5Gui.so",
                    "./" + DISTRO_DIR + "/lib/libQt5Network.so",
                    "./" + DISTRO_DIR + "/lib/libQt5Positioning.so",
                    "./" + DISTRO_DIR + "/lib/libQt5Qml.so",
                    "./" + DISTRO_DIR + "/lib/libQt5Quick.so",
                    "./" + DISTRO_DIR + "/lib/libQt5QuickControls2.so",
                    "./" + DISTRO_DIR + "/lib/libQt5QuickTemplates2.so",
                    "./" + DISTRO_DIR + "/lib/libQt5RemoteObjects.so",
                    "./" + DISTRO_DIR + "/lib/libQt5SerialPort.so",
                    "./" + DISTRO_DIR + "/lib/libQt5Svg.so",
                    "./" + DISTRO_DIR + "/lib/libQt5VirtualKeyboard.so",
                    "./" + DISTRO_DIR + "/lib/libQt5WaylandClient.so",
                    "./" + DISTRO_DIR + "/lib/libQt5WaylandCompositor.so",
                    "./" + DISTRO_DIR + "/lib/libQt5WebChannel.so",
                    "./" + DISTRO_DIR + "/lib/libQt5WebEngine.so",
                    "./" + DISTRO_DIR + "/lib/libQt5WebEngineCore.so",
                    "./" + DISTRO_DIR + "/lib/libQt5WebSockets.so",
                    "./" + DISTRO_DIR + "/lib/libQt5Widgets.so",
                    "./" + DISTRO_DIR + "/lib/libQt5XcbQpa.so",
                    "./" + DISTRO_DIR + "/lib/libicudata.so",
                    "./" + DISTRO_DIR + "/lib/libicui18n.so",
                    "./" + DISTRO_DIR + "/lib/libicuuc.so",
                    "./" + DISTRO_DIR + "/plugins/bearer/libqconnmanbearer.so",
                    "./" + DISTRO_DIR + "/plugins/bearer/libqgenericbearer.so",
                    "./" + DISTRO_DIR + "/plugins/bearer/libqnmbearer.so",
                    "./" + DISTRO_DIR + "/plugins/iconengines/libqsvgicon.so",
                    "./" + DISTRO_DIR + "/plugins/imageformats/libqgif.so",
                    "./" + DISTRO_DIR + "/plugins/imageformats/libqicns.so",
                    "./" + DISTRO_DIR + "/plugins/imageformats/libqico.so",
                    "./" + DISTRO_DIR + "/plugins/imageformats/libqjpeg.so",
                    "./" + DISTRO_DIR + "/plugins/imageformats/libqsvg.so",
                    "./" + DISTRO_DIR + "/plugins/imageformats/libqtga.so",
                    "./" + DISTRO_DIR + "/plugins/imageformats/libqtiff.so",
                    "./" + DISTRO_DIR + "/plugins/imageformats/libqwbmp.so",
                    "./" + DISTRO_DIR + "/plugins/imageformats/libqwebp.so",
                    "./" + DISTRO_DIR + "/plugins/platforminputcontexts/libcomposeplatforminputcontextplugin.so",
                    "./" + DISTRO_DIR + "/plugins/platforminputcontexts/libibusplatforminputcontextplugin.so",
                    "./" + DISTRO_DIR + "/plugins/platforminputcontexts/libqtvirtualkeyboardplugin.so",
                    "./" + DISTRO_DIR + "/plugins/platforms/libqeglfs.so",
                    "./" + DISTRO_DIR + "/plugins/platforms/libqlinuxfb.so",
                    "./" + DISTRO_DIR + "/plugins/platforms/libqminimal.so",
                    "./" + DISTRO_DIR + "/plugins/platforms/libqminimalegl.so",
                    "./" + DISTRO_DIR + "/plugins/platforms/libqoffscreen.so",
                    "./" + DISTRO_DIR + "/plugins/platforms/libqvnc.so",
                    "./" + DISTRO_DIR + "/plugins/platforms/libqwayland-egl.so",
                    "./" + DISTRO_DIR + "/plugins/platforms/libqwayland-generic.so",
                    "./" + DISTRO_DIR + "/plugins/platforms/libqwayland-xcomposite-egl.so",
                    "./" + DISTRO_DIR + "/plugins/platforms/libqwayland-xcomposite-glx.so",
                    "./" + DISTRO_DIR + "/plugins/platforms/libqwebgl.so",
                    "./" + DISTRO_DIR + "/plugins/platforms/libqxcb.so",
                    "./" + DISTRO_DIR + "/plugins/position/libqtposition_geoclue.so",
                    "./" + DISTRO_DIR + "/plugins/position/libqtposition_geoclue2.so",
                    "./" + DISTRO_DIR + "/plugins/position/libqtposition_positionpoll.so",
                    "./" + DISTRO_DIR + "/plugins/position/libqtposition_serialnmea.so",
                    "./" + DISTRO_DIR + "/plugins/virtualkeyboard/libqtvirtualkeyboard_hangul.so",
                    "./" + DISTRO_DIR + "/plugins/virtualkeyboard/libqtvirtualkeyboard_openwnn.so",
                    "./" + DISTRO_DIR + "/plugins/virtualkeyboard/libqtvirtualkeyboard_pinyin.so",
                    "./" + DISTRO_DIR + "/plugins/virtualkeyboard/libqtvirtualkeyboard_tcime.so",
                    "./" + DISTRO_DIR + "/plugins/virtualkeyboard/libqtvirtualkeyboard_thai.so",
                    "./" + DISTRO_DIR + "/plugins/wayland-decoration-client/libbradient.so",
                    "./" + DISTRO_DIR + "/plugins/wayland-graphics-integration-client/libdmabuf-server.so",
                    "./" + DISTRO_DIR + "/plugins/wayland-graphics-integration-client/libdrm-egl-server.so",
                    "./" + DISTRO_DIR + "/plugins/wayland-graphics-integration-client/libqt-plugin-wayland-egl.so",
                    "./" + DISTRO_DIR + "/plugins/wayland-graphics-integration-client/libshm-emulation-server.so",
                    "./" + DISTRO_DIR + "/plugins/wayland-graphics-integration-client/libxcomposite-egl.so",
                    "./" + DISTRO_DIR + "/plugins/wayland-graphics-integration-client/libxcomposite-glx.so",
                    "./" + DISTRO_DIR + "/plugins/wayland-graphics-integration-server/libdmabuf-server.so",
                    "./" + DISTRO_DIR + "/plugins/wayland-graphics-integration-server/libdrm-egl-server.so",
                    "./" + DISTRO_DIR + "/plugins/wayland-graphics-integration-server/liblinux-dmabuf-unstable-v1.so",
                    "./" + DISTRO_DIR + "/plugins/wayland-graphics-integration-server/libqt-plugin-wayland-egl.so",
                    "./" + DISTRO_DIR + "/plugins/wayland-graphics-integration-server/libshm-emulation-server.so",
                    "./" + DISTRO_DIR + "/plugins/wayland-graphics-integration-server/libwayland-eglstream-controller.so",
                    "./" + DISTRO_DIR + "/plugins/wayland-graphics-integration-server/libxcomposite-egl.so",
                    "./" + DISTRO_DIR + "/plugins/wayland-graphics-integration-server/libxcomposite-glx.so",
                    "./" + DISTRO_DIR + "/plugins/wayland-shell-integration/libfullscreen-shell-v1.so",
                    "./" + DISTRO_DIR + "/plugins/wayland-shell-integration/libivi-shell.so",
                    "./" + DISTRO_DIR + "/plugins/wayland-shell-integration/libwl-shell.so",
                    "./" + DISTRO_DIR + "/plugins/wayland-shell-integration/libxdg-shell-v5.so",
                    "./" + DISTRO_DIR + "/plugins/wayland-shell-integration/libxdg-shell-v6.so",
                    "./" + DISTRO_DIR + "/plugins/wayland-shell-integration/libxdg-shell.so",
                    "./" + DISTRO_DIR + "/plugins/xcbglintegrations/libqxcb-egl-integration.so",
                    "./" + DISTRO_DIR + "/plugins/xcbglintegrations/libqxcb-glx-integration.so",
                    "./" + DISTRO_DIR + "/qml/Qt/labs/folderlistmodel/libqmlfolderlistmodelplugin.so",
                    "./" + DISTRO_DIR + "/qml/Qt/labs/folderlistmodel/plugins.qmltypes",
                    "./" + DISTRO_DIR + "/qml/Qt/labs/folderlistmodel/qmldir",
                    "./" + DISTRO_DIR + "/qml/Qt/labs/settings/libqmlsettingsplugin.so",
                    "./" + DISTRO_DIR + "/qml/Qt/labs/settings/plugins.qmltypes",
                    "./" + DISTRO_DIR + "/qml/Qt/labs/settings/qmldir",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/Blend.qml",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/BrightnessContrast.qml",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/ColorOverlay.qml",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/Colorize.qml",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/ConicalGradient.qml",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/Desaturate.qml",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/DirectionalBlur.qml",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/Displace.qml",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/DropShadow.qml",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/FastBlur.qml",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/GammaAdjust.qml",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/GaussianBlur.qml",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/Glow.qml",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/HueSaturation.qml",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/InnerShadow.qml",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/LevelAdjust.qml",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/LinearGradient.qml",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/MaskedBlur.qml",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/OpacityMask.qml",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/RadialBlur.qml",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/RadialGradient.qml",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/RectangularGlow.qml",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/RecursiveBlur.qml",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/ThresholdMask.qml",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/ZoomBlur.qml",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/libqtgraphicaleffectsplugin.so",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/plugins.qmltypes",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/private/DropShadowBase.qml",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/private/DropShadowBase.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/private/FastGlow.qml",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/private/FastGlow.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/private/FastInnerShadow.qml",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/private/FastInnerShadow.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/private/FastMaskedBlur.qml",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/private/FastMaskedBlur.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/private/GaussianDirectionalBlur.qml",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/private/GaussianDirectionalBlur.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/private/GaussianGlow.qml",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/private/GaussianGlow.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/private/GaussianInnerShadow.qml",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/private/GaussianInnerShadow.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/private/GaussianMaskedBlur.qml",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/private/GaussianMaskedBlur.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/private/libqtgraphicaleffectsprivate.so",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/private/qmldir",
                    "./" + DISTRO_DIR + "/qml/QtGraphicalEffects/qmldir",
                    "./" + DISTRO_DIR + "/qml/QtQml/Models.2/libmodelsplugin.so",
                    "./" + DISTRO_DIR + "/qml/QtQml/Models.2/plugins.qmltypes",
                    "./" + DISTRO_DIR + "/qml/QtQml/Models.2/qmldir",
                    "./" + DISTRO_DIR + "/qml/QtQml/RemoteObjects/libqtqmlremoteobjects.so",
                    "./" + DISTRO_DIR + "/qml/QtQml/RemoteObjects/plugins.qmltypes",
                    "./" + DISTRO_DIR + "/qml/QtQml/RemoteObjects/qmldir",
                    "./" + DISTRO_DIR + "/qml/QtQml/StateMachine/libqtqmlstatemachine.so",
                    "./" + DISTRO_DIR + "/qml/QtQml/StateMachine/plugins.qmltypes",
                    "./" + DISTRO_DIR + "/qml/QtQml/StateMachine/qmldir",
                    "./" + DISTRO_DIR + "/qml/QtQml/plugins.qmltypes",
                    "./" + DISTRO_DIR + "/qml/QtQml/qmldir",
                    "./" + DISTRO_DIR + "/qml/QtQuick.2/libqtquick2plugin.so",
                    "./" + DISTRO_DIR + "/qml/QtQuick.2/plugins.qmltypes",
                    "./" + DISTRO_DIR + "/qml/QtQuick.2/qmldir",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/AbstractButton.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Action.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/ActionGroup.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/ApplicationWindow.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/BusyIndicator.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Button.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/ButtonGroup.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/CheckBox.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/CheckDelegate.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/ComboBox.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Container.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Control.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/DelayButton.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Dial.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Dialog.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/DialogButtonBox.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Drawer.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Frame.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/ApplicationWindow.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/BusyIndicator.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/Button.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/ButtonPanel.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/CheckBox.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/CheckDelegate.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/CheckIndicator.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/ComboBox.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/DelayButton.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/Dial.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/Dialog.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/DialogButtonBox.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/Drawer.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/Frame.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/GroupBox.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/ItemDelegate.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/Label.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/Menu.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/MenuBar.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/MenuBarItem.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/MenuItem.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/MenuSeparator.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/Page.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/PageIndicator.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/Pane.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/Popup.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/ProgressBar.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/RadioButton.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/RadioDelegate.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/RadioIndicator.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/RangeSlider.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/RoundButton.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/ScrollBar.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/ScrollIndicator.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/Slider.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/SliderGroove.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/SliderHandle.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/SpinBox.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/SplitView.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/SwipeDelegate.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/Switch.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/SwitchDelegate.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/SwitchIndicator.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/TabBar.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/TabButton.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/TextArea.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/TextField.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/ToolBar.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/ToolButton.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/ToolSeparator.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/ToolTip.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/Tumbler.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/libqtquickcontrols2fusionstyleplugin.so",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/plugins.qmltypes",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Fusion/qmldir",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/GroupBox.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/ApplicationWindow.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/BusyIndicator.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/Button.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/CheckBox.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/CheckDelegate.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/ComboBox.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/DelayButton.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/Dial.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/Dialog.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/DialogButtonBox.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/Drawer.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/Frame.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/GroupBox.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/ItemDelegate.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/Label.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/Menu.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/MenuItem.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/MenuSeparator.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/Page.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/PageIndicator.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/Pane.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/Popup.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/ProgressBar.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/RadioButton.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/RadioDelegate.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/RangeSlider.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/RoundButton.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/ScrollBar.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/ScrollIndicator.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/Slider.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/SpinBox.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/SplitView.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/StackView.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/SwipeDelegate.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/SwipeView.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/Switch.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/SwitchDelegate.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/TabBar.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/TabButton.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/TextArea.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/TextField.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/ToolBar.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/ToolButton.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/ToolSeparator.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/ToolTip.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/Tumbler.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/libqtquickcontrols2imaginestyleplugin.so",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/plugins.qmltypes",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Imagine/qmldir",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/ItemDelegate.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Label.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/ApplicationWindow.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/BoxShadow.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/BusyIndicator.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/Button.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/CheckBox.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/CheckDelegate.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/CheckIndicator.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/ComboBox.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/CursorDelegate.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/DelayButton.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/Dial.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/Dialog.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/DialogButtonBox.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/Drawer.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/ElevationEffect.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/Frame.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/GroupBox.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/ItemDelegate.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/Label.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/Menu.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/MenuBar.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/MenuBarItem.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/MenuItem.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/MenuSeparator.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/Page.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/PageIndicator.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/Pane.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/Popup.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/ProgressBar.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/RadioButton.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/RadioDelegate.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/RadioIndicator.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/RangeSlider.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/RectangularGlow.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/RoundButton.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/ScrollBar.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/ScrollIndicator.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/Slider.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/SliderHandle.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/SpinBox.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/SplitView.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/StackView.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/SwipeDelegate.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/SwipeView.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/Switch.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/SwitchDelegate.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/SwitchIndicator.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/TabBar.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/TabButton.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/TextArea.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/TextField.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/ToolBar.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/ToolButton.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/ToolSeparator.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/ToolTip.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/Tumbler.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/libqtquickcontrols2materialstyleplugin.so",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/plugins.qmltypes",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Material/qmldir",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Menu.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/MenuBar.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/MenuBarItem.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/MenuItem.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/MenuSeparator.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Page.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/PageIndicator.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Pane.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Popup.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/ProgressBar.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/RadioButton.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/RadioDelegate.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/RangeSlider.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/RoundButton.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/ScrollBar.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/ScrollIndicator.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/ScrollView.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Slider.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/SpinBox.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/SplitView.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/StackView.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/SwipeDelegate.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/SwipeView.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Switch.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/SwitchDelegate.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/TabBar.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/TabButton.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/TextArea.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/TextField.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/ToolBar.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/ToolButton.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/ToolSeparator.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/ToolTip.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Tumbler.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/ApplicationWindow.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/BusyIndicator.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/Button.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/CheckBox.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/CheckDelegate.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/CheckIndicator.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/ComboBox.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/DelayButton.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/Dial.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/Dialog.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/DialogButtonBox.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/Drawer.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/Frame.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/GroupBox.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/ItemDelegate.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/Label.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/Menu.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/MenuBar.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/MenuBarItem.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/MenuItem.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/MenuSeparator.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/Page.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/PageIndicator.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/Pane.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/Popup.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/ProgressBar.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/RadioButton.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/RadioDelegate.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/RadioIndicator.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/RangeSlider.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/RoundButton.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/ScrollBar.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/ScrollIndicator.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/Slider.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/SpinBox.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/SplitView.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/StackView.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/SwipeDelegate.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/Switch.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/SwitchDelegate.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/SwitchIndicator.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/TabBar.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/TabButton.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/TextArea.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/TextField.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/ToolBar.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/ToolButton.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/ToolSeparator.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/ToolTip.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/Tumbler.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/libqtquickcontrols2universalstyleplugin.so",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/plugins.qmltypes",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/Universal/qmldir",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/AbstractButtonSection.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/BusyIndicatorSpecifics.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/ButtonSection.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/ButtonSpecifics.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/CheckBoxSpecifics.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/CheckDelegateSpecifics.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/CheckSection.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/ComboBoxSpecifics.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/ContainerSection.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/ControlSection.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/ControlSpecifics.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/DelayButtonSpecifics.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/DialSpecifics.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/FrameSpecifics.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/GroupBoxSpecifics.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/ItemDelegateSection.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/ItemDelegateSpecifics.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/LabelSpecifics.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/PaddingSection.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/PageIndicatorSpecifics.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/PageSpecifics.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/PaneSection.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/PaneSpecifics.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/ProgressBarSpecifics.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/RadioButtonSpecifics.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/RadioDelegateSpecifics.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/RangeSliderSpecifics.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/RoundButtonSpecifics.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/ScrollViewSpecifics.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/SliderSpecifics.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/SpinBoxSpecifics.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/StackViewSpecifics.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/SwipeDelegateSpecifics.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/SwipeViewSpecifics.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/SwitchDelegateSpecifics.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/SwitchSpecifics.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/TabBarSpecifics.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/TabButtonSpecifics.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/TextAreaSpecifics.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/TextFieldSpecifics.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/ToolBarSpecifics.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/ToolButtonSpecifics.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/ToolSeparatorSpecifics.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/TumblerSpecifics.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/busyindicator-icon.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/busyindicator-icon16.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/busyindicator-icon@2x.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/button-icon.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/button-icon16.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/button-icon@2x.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/checkbox-icon.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/checkbox-icon16.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/checkbox-icon@2x.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/combobox-icon.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/combobox-icon16.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/combobox-icon@2x.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/delaybutton-icon.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/delaybutton-icon16.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/delaybutton-icon@2x.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/dial-icon.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/dial-icon16.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/dial-icon@2x.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/frame-icon.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/frame-icon16.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/frame-icon@2x.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/groupbox-icon.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/groupbox-icon16.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/groupbox-icon@2x.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/itemdelegate-icon.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/itemdelegate-icon16.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/itemdelegate-icon@2x.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/label-icon.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/label-icon16.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/label-icon@2x.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/page-icon.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/page-icon16.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/page-icon@2x.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/pageindicator-icon.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/pageindicator-icon16.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/pageindicator-icon@2x.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/pane-icon.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/pane-icon16.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/pane-icon@2x.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/progressbar-icon.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/progressbar-icon16.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/progressbar-icon@2x.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/radiobutton-icon.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/radiobutton-icon16.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/radiobutton-icon@2x.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/rangeslider-icon.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/rangeslider-icon16.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/rangeslider-icon@2x.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/roundbutton-icon.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/roundbutton-icon16.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/roundbutton-icon@2x.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/scrollview-icon.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/scrollview-icon16.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/scrollview-icon@2x.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/slider-icon.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/slider-icon16.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/slider-icon@2x.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/spinbox-icon.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/spinbox-icon16.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/spinbox-icon@2x.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/stackview-icon.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/stackview-icon16.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/stackview-icon@2x.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/swipeview-icon.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/swipeview-icon16.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/swipeview-icon@2x.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/switch-icon.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/switch-icon16.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/switch-icon@2x.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/textarea-icon.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/textarea-icon16.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/textarea-icon@2x.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/textfield-icon.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/textfield-icon16.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/textfield-icon@2x.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/toolbar-icon.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/toolbar-icon16.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/toolbar-icon@2x.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/toolbutton-icon.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/toolbutton-icon16.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/toolbutton-icon@2x.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/toolseparator-icon.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/toolseparator-icon16.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/toolseparator-icon@2x.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/tumbler-icon.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/tumbler-icon16.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/images/tumbler-icon@2x.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/designer/qtquickcontrols2.metainfo",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/libqtquickcontrols2plugin.so",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/plugins.qmltypes",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls.2/qmldir",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/ApplicationWindow.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/ApplicationWindow.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/BusyIndicator.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/BusyIndicator.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Button.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Button.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Calendar.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Calendar.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/CheckBox.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/CheckBox.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/ComboBox.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/ComboBox.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/GroupBox.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/GroupBox.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Label.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Label.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Menu.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Menu.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/MenuBar.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/MenuBar.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Private/AbstractCheckable.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Private/AbstractCheckable.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Private/BasicButton.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Private/BasicButton.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Private/BasicTableView.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Private/BasicTableView.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Private/CalendarHeaderModel.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Private/CalendarHeaderModel.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Private/CalendarUtils.js",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Private/CalendarUtils.jsc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Private/ColumnMenuContent.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Private/ColumnMenuContent.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Private/ContentItem.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Private/ContentItem.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Private/Control.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Private/Control.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Private/EditMenu.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Private/EditMenu.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Private/EditMenu_base.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Private/EditMenu_base.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Private/FastGlow.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Private/FastGlow.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Private/FocusFrame.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Private/FocusFrame.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Private/HoverButton.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Private/HoverButton.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Private/MenuContentItem.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Private/MenuContentItem.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Private/MenuContentScroller.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Private/MenuContentScroller.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Private/MenuItemSubControls.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Private/MenuItemSubControls.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Private/ModalPopupBehavior.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Private/ModalPopupBehavior.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Private/ScrollBar.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Private/ScrollBar.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Private/ScrollViewHelper.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Private/ScrollViewHelper.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Private/SourceProxy.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Private/SourceProxy.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Private/StackView.js",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Private/StackView.jsc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Private/StackViewSlideDelegate.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Private/StackViewSlideDelegate.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Private/Style.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Private/Style.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Private/SystemPaletteSingleton.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Private/SystemPaletteSingleton.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Private/TabBar.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Private/TabBar.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Private/TableViewItemDelegateLoader.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Private/TableViewItemDelegateLoader.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Private/TableViewSelection.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Private/TableViewSelection.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Private/TextHandle.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Private/TextHandle.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Private/TextInputWithHandles.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Private/TextInputWithHandles.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Private/TextSingleton.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Private/TextSingleton.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Private/ToolMenuButton.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Private/ToolMenuButton.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Private/TreeViewItemDelegateLoader.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Private/TreeViewItemDelegateLoader.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Private/qmldir",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Private/style.js",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Private/style.jsc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/ProgressBar.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/ProgressBar.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/RadioButton.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/RadioButton.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/ScrollView.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/ScrollView.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Slider.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Slider.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/SpinBox.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/SpinBox.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/SplitView.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/SplitView.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/StackView.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/StackView.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/StackViewDelegate.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/StackViewDelegate.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/StackViewTransition.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/StackViewTransition.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/StatusBar.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/StatusBar.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Base/ApplicationWindowStyle.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Base/ApplicationWindowStyle.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Base/BasicTableViewStyle.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Base/BasicTableViewStyle.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Base/BusyIndicatorStyle.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Base/BusyIndicatorStyle.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Base/ButtonStyle.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Base/ButtonStyle.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Base/CalendarStyle.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Base/CalendarStyle.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Base/CheckBoxStyle.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Base/CheckBoxStyle.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Base/CircularButtonStyle.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Base/CircularButtonStyle.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Base/CircularGaugeStyle.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Base/CircularGaugeStyle.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Base/CircularTickmarkLabelStyle.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Base/CircularTickmarkLabelStyle.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Base/ComboBoxStyle.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Base/ComboBoxStyle.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Base/CommonStyleHelper.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Base/CommonStyleHelper.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Base/DelayButtonStyle.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Base/DelayButtonStyle.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Base/DialStyle.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Base/DialStyle.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Base/FocusFrameStyle.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Base/FocusFrameStyle.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Base/GaugeStyle.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Base/GaugeStyle.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Base/GroupBoxStyle.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Base/GroupBoxStyle.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Base/HandleStyle.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Base/HandleStyle.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Base/HandleStyleHelper.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Base/HandleStyleHelper.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Base/MenuBarStyle.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Base/MenuBarStyle.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Base/MenuStyle.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Base/MenuStyle.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Base/PieMenuStyle.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Base/PieMenuStyle.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Base/ProgressBarStyle.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Base/ProgressBarStyle.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Base/RadioButtonStyle.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Base/RadioButtonStyle.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Base/ScrollViewStyle.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Base/ScrollViewStyle.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Base/SliderStyle.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Base/SliderStyle.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Base/SpinBoxStyle.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Base/SpinBoxStyle.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Base/StatusBarStyle.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Base/StatusBarStyle.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Base/StatusIndicatorStyle.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Base/StatusIndicatorStyle.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Base/SwitchStyle.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Base/SwitchStyle.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Base/TabViewStyle.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Base/TabViewStyle.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Base/TableViewStyle.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Base/TableViewStyle.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Base/TextAreaStyle.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Base/TextAreaStyle.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Base/TextFieldStyle.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Base/TextFieldStyle.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Base/ToggleButtonStyle.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Base/ToggleButtonStyle.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Base/ToolBarStyle.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Base/ToolBarStyle.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Base/ToolButtonStyle.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Base/ToolButtonStyle.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Base/TreeViewStyle.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Base/TreeViewStyle.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Base/TumblerStyle.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Base/TumblerStyle.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Base/images/arrow-down.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Base/images/arrow-down@2x.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Base/images/arrow-left.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Base/images/arrow-left@2x.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Base/images/arrow-right.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Base/images/arrow-right@2x.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Base/images/arrow-up.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Base/images/arrow-up@2x.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Base/images/button.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Base/images/button_down.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Base/images/check.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Base/images/check@2x.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Base/images/editbox.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Base/images/focusframe.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Base/images/groupbox.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Base/images/header.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Base/images/knob.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Base/images/leftanglearrow.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Base/images/needle.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Base/images/progress-indeterminate.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Base/images/rightanglearrow.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Base/images/scrollbar-handle-horizontal.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Base/images/scrollbar-handle-transient.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Base/images/scrollbar-handle-vertical.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Base/images/slider-groove.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Base/images/slider-handle.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Base/images/spinner_large.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Base/images/spinner_medium.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Base/images/spinner_small.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Base/images/tab.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Base/images/tab_selected.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Desktop/ApplicationWindowStyle.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Desktop/ApplicationWindowStyle.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Desktop/BusyIndicatorStyle.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Desktop/BusyIndicatorStyle.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Desktop/ButtonStyle.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Desktop/ButtonStyle.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Desktop/CalendarStyle.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Desktop/CalendarStyle.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Desktop/CheckBoxStyle.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Desktop/CheckBoxStyle.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Desktop/ComboBoxStyle.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Desktop/ComboBoxStyle.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Desktop/FocusFrameStyle.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Desktop/FocusFrameStyle.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Desktop/GroupBoxStyle.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Desktop/GroupBoxStyle.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Desktop/MenuBarStyle.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Desktop/MenuBarStyle.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Desktop/MenuStyle.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Desktop/MenuStyle.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Desktop/ProgressBarStyle.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Desktop/ProgressBarStyle.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Desktop/RadioButtonStyle.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Desktop/RadioButtonStyle.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Desktop/RowItemSingleton.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Desktop/RowItemSingleton.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Desktop/ScrollViewStyle.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Desktop/ScrollViewStyle.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Desktop/SliderStyle.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Desktop/SliderStyle.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Desktop/SpinBoxStyle.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Desktop/SpinBoxStyle.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Desktop/StatusBarStyle.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Desktop/StatusBarStyle.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Desktop/SwitchStyle.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Desktop/SwitchStyle.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Desktop/TabViewStyle.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Desktop/TabViewStyle.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Desktop/TableViewStyle.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Desktop/TableViewStyle.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Desktop/TextAreaStyle.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Desktop/TextAreaStyle.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Desktop/TextFieldStyle.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Desktop/TextFieldStyle.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Desktop/ToolBarStyle.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Desktop/ToolBarStyle.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Desktop/ToolButtonStyle.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Desktop/ToolButtonStyle.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Desktop/TreeViewStyle.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Desktop/TreeViewStyle.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Desktop/qmldir",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Flat/libqtquickextrasflatplugin.so",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Flat/plugins.qmltypes",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/Flat/qmldir",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Styles/qmldir",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Switch.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Switch.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Tab.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/Tab.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/TabView.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/TabView.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/TableView.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/TableView.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/TableViewColumn.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/TableViewColumn.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/TextArea.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/TextArea.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/TextField.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/TextField.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/ToolBar.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/ToolBar.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/ToolButton.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/ToolButton.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/TreeView.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/TreeView.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/libqtquickcontrolsplugin.so",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/plugins.qmltypes",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Controls/qmldir",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Dialogs/DefaultColorDialog.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Dialogs/DefaultColorDialog.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Dialogs/DefaultDialogWrapper.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Dialogs/DefaultDialogWrapper.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Dialogs/DefaultFileDialog.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Dialogs/DefaultFileDialog.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Dialogs/DefaultFontDialog.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Dialogs/DefaultFontDialog.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Dialogs/DefaultMessageDialog.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Dialogs/DefaultMessageDialog.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Dialogs/Private/libdialogsprivateplugin.so",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Dialogs/Private/plugins.qmltypes",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Dialogs/Private/qmldir",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Dialogs/WidgetColorDialog.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Dialogs/WidgetColorDialog.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Dialogs/WidgetFileDialog.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Dialogs/WidgetFileDialog.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Dialogs/WidgetFontDialog.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Dialogs/WidgetFontDialog.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Dialogs/WidgetMessageDialog.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Dialogs/WidgetMessageDialog.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Dialogs/images/checkers.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Dialogs/images/checkmark.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Dialogs/images/copy.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Dialogs/images/critical.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Dialogs/images/crosshairs.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Dialogs/images/information.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Dialogs/images/question.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Dialogs/images/slider_handle.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Dialogs/images/sunken_frame.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Dialogs/images/warning.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Dialogs/images/window_border.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Dialogs/libdialogplugin.so",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Dialogs/plugins.qmltypes",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Dialogs/qml/ColorSlider.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Dialogs/qml/ColorSlider.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Dialogs/qml/DefaultWindowDecoration.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Dialogs/qml/DefaultWindowDecoration.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Dialogs/qml/IconButtonStyle.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Dialogs/qml/IconButtonStyle.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Dialogs/qml/IconGlyph.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Dialogs/qml/IconGlyph.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Dialogs/qml/icons.ttf",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Dialogs/qml/qmldir",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Dialogs/qmldir",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Extras/CircularGauge.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Extras/CircularGauge.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Extras/DelayButton.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Extras/DelayButton.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Extras/Dial.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Extras/Dial.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Extras/Gauge.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Extras/Gauge.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Extras/PieMenu.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Extras/PieMenu.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Extras/Private/CircularButton.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Extras/Private/CircularButton.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Extras/Private/CircularButtonStyleHelper.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Extras/Private/CircularButtonStyleHelper.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Extras/Private/CircularTickmarkLabel.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Extras/Private/CircularTickmarkLabel.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Extras/Private/Handle.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Extras/Private/Handle.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Extras/Private/PieMenuIcon.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Extras/Private/PieMenuIcon.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Extras/Private/TextSingleton.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Extras/Private/TextSingleton.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Extras/Private/qmldir",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Extras/StatusIndicator.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Extras/StatusIndicator.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Extras/ToggleButton.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Extras/ToggleButton.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Extras/Tumbler.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Extras/Tumbler.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Extras/TumblerColumn.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Extras/TumblerColumn.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Extras/designer/CircularGaugeSpecifics.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Extras/designer/CircularGaugeSpecifics.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Extras/designer/DelayButtonSpecifics.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Extras/designer/DelayButtonSpecifics.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Extras/designer/DialSpecifics.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Extras/designer/DialSpecifics.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Extras/designer/GaugeSpecifics.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Extras/designer/GaugeSpecifics.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Extras/designer/PictureSpecifics.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Extras/designer/PictureSpecifics.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Extras/designer/PieMenuSpecifics.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Extras/designer/PieMenuSpecifics.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Extras/designer/StatusIndicatorSpecifics.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Extras/designer/StatusIndicatorSpecifics.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Extras/designer/ToggleButtonSpecifics.qml",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Extras/designer/ToggleButtonSpecifics.qmlc",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Extras/designer/images/circulargauge-icon.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Extras/designer/images/circulargauge-icon16.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Extras/designer/images/delaybutton-icon.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Extras/designer/images/delaybutton-icon16.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Extras/designer/images/dial-icon.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Extras/designer/images/dial-icon16.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Extras/designer/images/gauge-icon.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Extras/designer/images/gauge-icon16.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Extras/designer/images/picture-icon.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Extras/designer/images/picture-icon16.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Extras/designer/images/piemenu-icon.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Extras/designer/images/piemenu-icon16.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Extras/designer/images/statusindicator-icon.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Extras/designer/images/statusindicator-icon16.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Extras/designer/images/togglebutton-icon.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Extras/designer/images/togglebutton-icon16.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Extras/designer/images/tumbler-icon.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Extras/designer/images/tumbler-icon16.png",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Extras/designer/qtquickextras.metainfo",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Extras/libqtquickextrasplugin.so",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Extras/plugins.qmltypes",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Extras/qmldir",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Layouts/libqquicklayoutsplugin.so",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Layouts/plugins.qmltypes",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Layouts/qmldir",
                    "./" + DISTRO_DIR + "/qml/QtQuick/PrivateWidgets/libwidgetsplugin.so",
                    "./" + DISTRO_DIR + "/qml/QtQuick/PrivateWidgets/plugins.qmltypes",
                    "./" + DISTRO_DIR + "/qml/QtQuick/PrivateWidgets/qmldir",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Templates.2/libqtquicktemplates2plugin.so",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Templates.2/plugins.qmltypes",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Templates.2/qmldir",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Window.2/libwindowplugin.so",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Window.2/plugins.qmltypes",
                    "./" + DISTRO_DIR + "/qml/QtQuick/Window.2/qmldir",
                    "./" + DISTRO_DIR + "/qml/QtWebEngine/Controls1Delegates/AlertDialog.qml",
                    "./" + DISTRO_DIR + "/qml/QtWebEngine/Controls1Delegates/AuthenticationDialog.qml",
                    "./" + DISTRO_DIR + "/qml/QtWebEngine/Controls1Delegates/ColorDialog.qml",
                    "./" + DISTRO_DIR + "/qml/QtWebEngine/Controls1Delegates/ConfirmDialog.qml",
                    "./" + DISTRO_DIR + "/qml/QtWebEngine/Controls1Delegates/FilePicker.qml",
                    "./" + DISTRO_DIR + "/qml/QtWebEngine/Controls1Delegates/Menu.qml",
                    "./" + DISTRO_DIR + "/qml/QtWebEngine/Controls1Delegates/MenuItem.qml",
                    "./" + DISTRO_DIR + "/qml/QtWebEngine/Controls1Delegates/MenuSeparator.qml",
                    "./" + DISTRO_DIR + "/qml/QtWebEngine/Controls1Delegates/PromptDialog.qml",
                    "./" + DISTRO_DIR + "/qml/QtWebEngine/Controls1Delegates/ToolTip.qml",
                    "./" + DISTRO_DIR + "/qml/QtWebEngine/Controls1Delegates/TouchHandle.qml",
                    "./" + DISTRO_DIR + "/qml/QtWebEngine/Controls1Delegates/TouchSelectionMenu.qml",
                    "./" + DISTRO_DIR + "/qml/QtWebEngine/Controls1Delegates/qmldir",
                    "./" + DISTRO_DIR + "/qml/QtWebEngine/Controls2Delegates/AlertDialog.qml",
                    "./" + DISTRO_DIR + "/qml/QtWebEngine/Controls2Delegates/AuthenticationDialog.qml",
                    "./" + DISTRO_DIR + "/qml/QtWebEngine/Controls2Delegates/ConfirmDialog.qml",
                    "./" + DISTRO_DIR + "/qml/QtWebEngine/Controls2Delegates/Menu.qml",
                    "./" + DISTRO_DIR + "/qml/QtWebEngine/Controls2Delegates/MenuItem.qml",
                    "./" + DISTRO_DIR + "/qml/QtWebEngine/Controls2Delegates/MenuSeparator.qml",
                    "./" + DISTRO_DIR + "/qml/QtWebEngine/Controls2Delegates/PromptDialog.qml",
                    "./" + DISTRO_DIR + "/qml/QtWebEngine/Controls2Delegates/ToolTip.qml",
                    "./" + DISTRO_DIR + "/qml/QtWebEngine/Controls2Delegates/information.png",
                    "./" + DISTRO_DIR + "/qml/QtWebEngine/Controls2Delegates/qmldir",
                    "./" + DISTRO_DIR + "/qml/QtWebEngine/Controls2Delegates/question.png",
                    "./" + DISTRO_DIR + "/qml/QtWebEngine/libqtwebengineplugin.so",
                    "./" + DISTRO_DIR + "/qml/QtWebEngine/plugins.qmltypes",
                    "./" + DISTRO_DIR + "/qml/QtWebEngine/qmldir",
                    "./" + DISTRO_DIR + "/quicknanobrowser.sh",
                    "./" + DISTRO_DIR + "/resources/icudtl.dat",
                    "./" + DISTRO_DIR + "/resources/qtwebengine_devtools_resources.pak",
                    "./" + DISTRO_DIR + "/resources/qtwebengine_resources.pak",
                    "./" + DISTRO_DIR + "/resources/qtwebengine_resources_100p.pak",
                    "./" + DISTRO_DIR + "/resources/qtwebengine_resources_200p.pak",
                    "./" + DISTRO_DIR + "/translations/qtbase_ar.qm",
                    "./" + DISTRO_DIR + "/translations/qtbase_bg.qm",
                    "./" + DISTRO_DIR + "/translations/qtbase_ca.qm",
                    "./" + DISTRO_DIR + "/translations/qtbase_cs.qm",
                    "./" + DISTRO_DIR + "/translations/qtbase_da.qm",
                    "./" + DISTRO_DIR + "/translations/qtbase_de.qm",
                    "./" + DISTRO_DIR + "/translations/qtbase_en.qm",
                    "./" + DISTRO_DIR + "/translations/qtbase_es.qm",
                    "./" + DISTRO_DIR + "/translations/qtbase_fi.qm",
                    "./" + DISTRO_DIR + "/translations/qtbase_fr.qm",
                    "./" + DISTRO_DIR + "/translations/qtbase_gd.qm",
                    "./" + DISTRO_DIR + "/translations/qtbase_he.qm",
                    "./" + DISTRO_DIR + "/translations/qtbase_hu.qm",
                    "./" + DISTRO_DIR + "/translations/qtbase_it.qm",
                    "./" + DISTRO_DIR + "/translations/qtbase_ja.qm",
                    "./" + DISTRO_DIR + "/translations/qtbase_ko.qm",
                    "./" + DISTRO_DIR + "/translations/qtbase_lv.qm",
                    "./" + DISTRO_DIR + "/translations/qtbase_pl.qm",
                    "./" + DISTRO_DIR + "/translations/qtbase_ru.qm",
                    "./" + DISTRO_DIR + "/translations/qtbase_sk.qm",
                    "./" + DISTRO_DIR + "/translations/qtbase_uk.qm",
                    "./" + DISTRO_DIR + "/translations/qtbase_zh_TW.qm",
                    "./" + DISTRO_DIR + "/translations/qtdeclarative_bg.qm",
                    "./" + DISTRO_DIR + "/translations/qtdeclarative_da.qm",
                    "./" + DISTRO_DIR + "/translations/qtdeclarative_de.qm",
                    "./" + DISTRO_DIR + "/translations/qtdeclarative_en.qm",
                    "./" + DISTRO_DIR + "/translations/qtdeclarative_es.qm",
                    "./" + DISTRO_DIR + "/translations/qtdeclarative_fi.qm",
                    "./" + DISTRO_DIR + "/translations/qtdeclarative_fr.qm",
                    "./" + DISTRO_DIR + "/translations/qtdeclarative_hu.qm",
                    "./" + DISTRO_DIR + "/translations/qtdeclarative_ja.qm",
                    "./" + DISTRO_DIR + "/translations/qtdeclarative_ko.qm",
                    "./" + DISTRO_DIR + "/translations/qtdeclarative_lv.qm",
                    "./" + DISTRO_DIR + "/translations/qtdeclarative_pl.qm",
                    "./" + DISTRO_DIR + "/translations/qtdeclarative_ru.qm",
                    "./" + DISTRO_DIR + "/translations/qtdeclarative_sk.qm",
                    "./" + DISTRO_DIR + "/translations/qtdeclarative_uk.qm",
                    "./" + DISTRO_DIR + "/translations/qtserialport_de.qm",
                    "./" + DISTRO_DIR + "/translations/qtserialport_en.qm",
                    "./" + DISTRO_DIR + "/translations/qtserialport_es.qm",
                    "./" + DISTRO_DIR + "/translations/qtserialport_ja.qm",
                    "./" + DISTRO_DIR + "/translations/qtserialport_ko.qm",
                    "./" + DISTRO_DIR + "/translations/qtserialport_pl.qm",
                    "./" + DISTRO_DIR + "/translations/qtserialport_ru.qm",
                    "./" + DISTRO_DIR + "/translations/qtserialport_uk.qm",
                    "./" + DISTRO_DIR + "/translations/qtwebengine_de.qm",
                    "./" + DISTRO_DIR + "/translations/qtwebengine_en.qm",
                    "./" + DISTRO_DIR + "/translations/qtwebengine_es.qm",
                    "./" + DISTRO_DIR + "/translations/qtwebengine_ko.qm",
                    "./" + DISTRO_DIR + "/translations/qtwebengine_locales/am.pak",
                    "./" + DISTRO_DIR + "/translations/qtwebengine_locales/ar.pak",
                    "./" + DISTRO_DIR + "/translations/qtwebengine_locales/bg.pak",
                    "./" + DISTRO_DIR + "/translations/qtwebengine_locales/bn.pak",
                    "./" + DISTRO_DIR + "/translations/qtwebengine_locales/ca.pak",
                    "./" + DISTRO_DIR + "/translations/qtwebengine_locales/cs.pak",
                    "./" + DISTRO_DIR + "/translations/qtwebengine_locales/da.pak",
                    "./" + DISTRO_DIR + "/translations/qtwebengine_locales/de.pak",
                    "./" + DISTRO_DIR + "/translations/qtwebengine_locales/el.pak",
                    "./" + DISTRO_DIR + "/translations/qtwebengine_locales/en-GB.pak",
                    "./" + DISTRO_DIR + "/translations/qtwebengine_locales/en-US.pak",
                    "./" + DISTRO_DIR + "/translations/qtwebengine_locales/es-419.pak",
                    "./" + DISTRO_DIR + "/translations/qtwebengine_locales/es.pak",
                    "./" + DISTRO_DIR + "/translations/qtwebengine_locales/et.pak",
                    "./" + DISTRO_DIR + "/translations/qtwebengine_locales/fa.pak",
                    "./" + DISTRO_DIR + "/translations/qtwebengine_locales/fi.pak",
                    "./" + DISTRO_DIR + "/translations/qtwebengine_locales/fil.pak",
                    "./" + DISTRO_DIR + "/translations/qtwebengine_locales/fr.pak",
                    "./" + DISTRO_DIR + "/translations/qtwebengine_locales/gu.pak",
                    "./" + DISTRO_DIR + "/translations/qtwebengine_locales/he.pak",
                    "./" + DISTRO_DIR + "/translations/qtwebengine_locales/hi.pak",
                    "./" + DISTRO_DIR + "/translations/qtwebengine_locales/hr.pak",
                    "./" + DISTRO_DIR + "/translations/qtwebengine_locales/hu.pak",
                    "./" + DISTRO_DIR + "/translations/qtwebengine_locales/id.pak",
                    "./" + DISTRO_DIR + "/translations/qtwebengine_locales/it.pak",
                    "./" + DISTRO_DIR + "/translations/qtwebengine_locales/ja.pak",
                    "./" + DISTRO_DIR + "/translations/qtwebengine_locales/kn.pak",
                    "./" + DISTRO_DIR + "/translations/qtwebengine_locales/ko.pak",
                    "./" + DISTRO_DIR + "/translations/qtwebengine_locales/lt.pak",
                    "./" + DISTRO_DIR + "/translations/qtwebengine_locales/lv.pak",
                    "./" + DISTRO_DIR + "/translations/qtwebengine_locales/ml.pak",
                    "./" + DISTRO_DIR + "/translations/qtwebengine_locales/mr.pak",
                    "./" + DISTRO_DIR + "/translations/qtwebengine_locales/ms.pak",
                    "./" + DISTRO_DIR + "/translations/qtwebengine_locales/nb.pak",
                    "./" + DISTRO_DIR + "/translations/qtwebengine_locales/nl.pak",
                    "./" + DISTRO_DIR + "/translations/qtwebengine_locales/pl.pak",
                    "./" + DISTRO_DIR + "/translations/qtwebengine_locales/pt-BR.pak",
                    "./" + DISTRO_DIR + "/translations/qtwebengine_locales/pt-PT.pak",
                    "./" + DISTRO_DIR + "/translations/qtwebengine_locales/ro.pak",
                    "./" + DISTRO_DIR + "/translations/qtwebengine_locales/ru.pak",
                    "./" + DISTRO_DIR + "/translations/qtwebengine_locales/sk.pak",
                    "./" + DISTRO_DIR + "/translations/qtwebengine_locales/sl.pak",
                    "./" + DISTRO_DIR + "/translations/qtwebengine_locales/sr.pak",
                    "./" + DISTRO_DIR + "/translations/qtwebengine_locales/sv.pak",
                    "./" + DISTRO_DIR + "/translations/qtwebengine_locales/sw.pak",
                    "./" + DISTRO_DIR + "/translations/qtwebengine_locales/ta.pak",
                    "./" + DISTRO_DIR + "/translations/qtwebengine_locales/te.pak",
                    "./" + DISTRO_DIR + "/translations/qtwebengine_locales/th.pak",
                    "./" + DISTRO_DIR + "/translations/qtwebengine_locales/tr.pak",
                    "./" + DISTRO_DIR + "/translations/qtwebengine_locales/uk.pak",
                    "./" + DISTRO_DIR + "/translations/qtwebengine_locales/vi.pak",
                    "./" + DISTRO_DIR + "/translations/qtwebengine_locales/zh-CN.pak",
                    "./" + DISTRO_DIR + "/translations/qtwebengine_locales/zh-TW.pak",
                    "./" + DISTRO_DIR + "/translations/qtwebengine_pl.qm",
                    "./" + DISTRO_DIR + "/translations/qtwebengine_ru.qm",
                    "./" + DISTRO_DIR + "/translations/qtwebengine_uk.qm"
                });
    return Tree;
}
